#include "advancewirellesssensor.h"

AdvanceWirellessSensor::AdvanceWirellessSensor(ManageSerialPort *_serialPort, Sensor_l _sensor, QWidget *parent):
    QWidget(parent)
{

    serialCom = _serialPort;
    sensor=_sensor;
    read_active=false;
    write_active=false;
    flag_timeout=false;
    program_active=false;
    save_file_active=false;
    settingFile = new QSettings("config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/ADV_W_CONFIG_WIN_X")&&
            settingFile->contains("WINDOW_POS/ADV_W_CONFIG_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/ADV_W_CONFIG_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/ADV_W_CONFIG_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    timer1= new QTimer(this);
    timer1->connect(timer1,SIGNAL(timeout()),this,SLOT(each_sec()));
    timer1->stop();
    timer2= new QTimer(this);
    timer2->connect(timer2,SIGNAL(timeout()),this,SLOT(timeout()));
    timer2->stop();
    setWindowIcon(QIcon("icon/radio.png"));
    setWindowTitle("Wireless Sensor Advanced Configuration");
    setWindowModality(Qt::ApplicationModal);
    fillWindow();
}

void AdvanceWirellessSensor::fillWindow()
{
    setMinimumWidth(200);
    setMinimumHeight(200);
    // building frame section
    sensor_sn= new QLabel(QString::number(sensor.serialNb,16).rightJustified(4,'0').toUpper());
        QLabel *serialNbLineLabel = new QLabel(this);
        serialNbLineLabel->setText("Serial Number :");
        QHBoxLayout *serialNbLayout = new QHBoxLayout;
        serialNbLayout->addWidget(serialNbLineLabel);
        serialNbLayout->addWidget(sensor_sn);
        serialNbLayout->setAlignment(Qt::AlignLeft);

    QGroupBox *readGroup = new QGroupBox("Read Data");
        // building frame section
        addressLine= new QLineEdit(this);
            addressLine->setText("0000");   // default = external memory
            QLabel *addressLineLabel = new QLabel(this);
            addressLineLabel->setText("Address :");
            QVBoxLayout *addressLayout = new QVBoxLayout;
            addressLayout->addWidget(addressLineLabel);
            addressLayout->addWidget(addressLine);
        select_memory=new QComboBox(this);
            select_memory->addItem("EEPROM");
            select_memory->addItem("RAM");
            QLabel *select_memoryLabel = new QLabel(this);
            select_memoryLabel->setText("Type of memory :");
            QVBoxLayout *select_memoryLayout = new QVBoxLayout;
            select_memoryLayout->addWidget(select_memoryLabel);
            select_memoryLayout->addWidget(select_memory);


        sendReadOrderButton = new QPushButton("Read Data");
        connect(sendReadOrderButton, SIGNAL(clicked()), SLOT(ReadClicked()));

        QHBoxLayout *layoutFabricTrame = new QHBoxLayout;
           // layoutFabricTrame->addLayout(serialNbLayout);
            layoutFabricTrame->addLayout(addressLayout);
            layoutFabricTrame->addLayout(select_memoryLayout);
            //layoutFabricTrame->addWidget(sendReadOrderButton);


        QHBoxLayout *layoutReceivedRead = new QHBoxLayout;
            dataReadDisplay=new QLineEdit();
            dataReadDisplay->setReadOnly(true);
        QLabel *dataReadLineLabel = new QLabel(this);
            dataReadLineLabel->setText("Data :");
            layoutReceivedRead->addWidget(dataReadLineLabel);
            layoutReceivedRead->addWidget(dataReadDisplay);

        QHBoxLayout *layoutFile = new QHBoxLayout;
            record_file = new QPushButton("Save data to file");
            layoutFile->addWidget(record_file);
            connect(record_file,SIGNAL(clicked()),this,SLOT(SaveFileClicked()));

        QVBoxLayout *layoutReadData = new QVBoxLayout;
            layoutReadData->addLayout(layoutFabricTrame);
            layoutReadData->addLayout(layoutReceivedRead);
            layoutReadData->addWidget(sendReadOrderButton);
            layoutReadData->addLayout(layoutFile);

            readGroup->setLayout(layoutReadData);
    //end readGroup

    //section write
    QGroupBox *writeGroup = new QGroupBox("Write Data in EEPROM");
        QVBoxLayout *writeDataLayout = new QVBoxLayout;
            addressLineWrite= new QLineEdit(this);
                addressLineWrite->setText("0000");   // default
                QLabel *addressLineWriteLabel = new QLabel(this);
                addressLineWriteLabel->setText("Address :");
                QVBoxLayout *addressWriteLayout = new QVBoxLayout;
                addressWriteLayout->addWidget(addressLineWriteLabel);
                addressWriteLayout->addWidget(addressLineWrite);

            reset_chip=new QCheckBox(this);
                QLabel *reset_chipLabel = new QLabel(this);
                reset_chipLabel->setText("Reset Chip :");
                QVBoxLayout *reset_chipLayout = new QVBoxLayout;
                reset_chipLayout->addWidget(reset_chipLabel);
                reset_chipLayout->addWidget(reset_chip);

            valueToWrite = new QLineEdit(this);
                valueToWrite->setText("00");
                QLabel *valueToWriteLabel = new QLabel(this);
                valueToWriteLabel->setText("Value to write :");
                QVBoxLayout *valueToWriteLayout = new QVBoxLayout;
                valueToWriteLayout->addWidget(valueToWriteLabel);
                valueToWriteLayout->addWidget(valueToWrite);

            writeButton = new QPushButton("Write Data");
            connect(writeButton, SIGNAL(clicked()), SLOT(WriteClicked()));     //TODO

            QHBoxLayout *writeFrameLayout = new QHBoxLayout;
                writeFrameLayout->addLayout(addressWriteLayout);
                writeFrameLayout->addLayout(reset_chipLayout);
                writeFrameLayout->addLayout(valueToWriteLayout);
               // writeFrameLayout->addWidget(writeButton);
        writeDataLayout->addLayout(writeFrameLayout);
        writeDataLayout->addWidget(writeButton);
        writeGroup->setLayout(writeDataLayout);
    //end writeGroup

    QGroupBox *writfirmwareGroup = new QGroupBox("Update Sensor Firmware");
        QVBoxLayout *writfirmVLayout0 = new QVBoxLayout;
        QHBoxLayout *writfirmHLayout0 = new QHBoxLayout;
        writfirmsensor = new QPushButton("Upload file to sensor");
        connect(writfirmsensor,SIGNAL(clicked()),this,SLOT(ProgramClicked()));
        QHBoxLayout *writfirmHLayout1 = new QHBoxLayout;
        write_firm_sensor_status = new QLabel(this);
        write_firm_sensor_status->setText(" ");
        writfirmHLayout0->addWidget(writfirmsensor);
        writfirmHLayout1->addWidget(write_firm_sensor_status);
        writfirmVLayout0->addLayout(writfirmHLayout0);
        writfirmVLayout0->addLayout(writfirmHLayout1);
        writfirmwareGroup->setLayout(writfirmVLayout0);

    QVBoxLayout* layoutTotal = new QVBoxLayout;
    layoutTotal->addLayout(serialNbLayout);
    layoutTotal->addWidget(readGroup);
    layoutTotal->addWidget(writeGroup);
    layoutTotal->addWidget(writfirmwareGroup);
    layoutTotal->setAlignment(Qt::AlignTop);
    setLayout(layoutTotal);
    setWindowModality(Qt::ApplicationModal);
}

AdvanceWirellessSensor::~AdvanceWirellessSensor(){

}

/*
void AdvanceWirellessSensor::closeEvent(QCloseEvent *event)
 {
    setVisible(false);
    emit AdvanceClosed();
}*/

void AdvanceWirellessSensor::ProgramClicked(){
    if(!program_active){
        writeButton->setEnabled(false);
        sendReadOrderButton->setEnabled(false);
        record_file->setEnabled(false);
        timer1->stop();
        timer1->start(500);
        toggle=true;
        program_active=true;
        writfirmsensor->setText("Wait for sensor, Press to interrupt");
        writfirmsensor->setStyleSheet("font-weight: bold;color: red");
        program_sensor();
    }
    else {
        writeButton->setEnabled(true);
        sendReadOrderButton->setEnabled(true);
        record_file->setEnabled(true);
        timer1->stop();
        program_active=false;
        writfirmsensor->setText("Upload file to sensor");
        writfirmsensor->setStyleSheet("color: black");
    }

}

void AdvanceWirellessSensor::SaveFileClicked(){
    if(!save_file_active){
        sendReadOrderButton->setEnabled(false);
        writeButton->setEnabled(false);
        writfirmsensor->setEnabled(false);
        timer1->stop();
        timer1->start(500);
        toggle=true;
        save_file_active=true;
        record_file->setText("Wait for sensor, Press to interrupt");
        record_file->setStyleSheet("font-weight: bold;color: red");
        QString filename_tmp="";
        if(settingFile->contains("ADV_CONFIG_W/DIR_HEX_FILE")){
           filename_tmp=(settingFile->value("ADV_CONFIG_W/DIR_HEX_FILE").toString());
        }
        QString filename=QFileDialog::getSaveFileName(this,"Choose Filename",filename_tmp, "*.hex");
        settingFile->setValue("ADV_CONFIG_W/DIR_HEX_FILE",QVariant(filename));
        sav_file= new QFile(filename);
        if(!sav_file->open(QFile::WriteOnly | QFile::Truncate)){
            QMessageBox::critical(this,"Error Opening File","The file can't be opened");
            return;
        }
        SendAction();
    }
    else {
        sendReadOrderButton->setEnabled(true);
        writeButton->setEnabled(true);
        writfirmsensor->setEnabled(true);
        timer1->stop();
        save_file_active=false;
        record_file->setText("Save data to file");
        record_file->setStyleSheet("color: black");
    }

}

void AdvanceWirellessSensor::ReadClicked(){
    if(!read_active){
        writeButton->setEnabled(false);
        writfirmsensor->setEnabled(false);
        record_file->setEnabled(false);
        timer1->stop();
        timer1->start(500);
        toggle=true;
        read_active=true;
        sendReadOrderButton->setText("Wait for sensor, Press to interrupt");
        sendReadOrderButton->setStyleSheet("font-weight: bold;color: red");
        SendAction();
    }
    else {
        writeButton->setEnabled(true);
        writfirmsensor->setEnabled(true);
        record_file->setEnabled(true);
        timer1->stop();
        read_active=false;
        sendReadOrderButton->setText("Read Data");
        sendReadOrderButton->setStyleSheet("color: black");
    }

}

void AdvanceWirellessSensor::WriteClicked(){
    if(!write_active){
        sendReadOrderButton->setEnabled(false);
        writfirmsensor->setEnabled(false);
        record_file->setEnabled(false);
        timer1->stop();
        timer1->start(500);
        toggle=true;
        write_active=true;
        writeButton->setText("Wait for sensor, Press to interrupt");
        writeButton->setStyleSheet("font-weight: bold;color: red");
        SendAction();
    }
    else {
        sendReadOrderButton->setEnabled(true);
        writfirmsensor->setEnabled(true);
        record_file->setEnabled(true);
        write_active=false;
        timer1->stop();
        writeButton->setText("Write Data");
        writeButton->setStyleSheet("color: black");
    }
}


void AdvanceWirellessSensor::timeout(){
    flag_timeout=true;
    timer2->stop();
}


void AdvanceWirellessSensor::each_sec(){
    if(read_active||write_active||program_active||save_file_active)
    {
        if(!toggle){
        if(read_active) sendReadOrderButton->setText("Wait for sensor, Press to interrupt");
        if(write_active) writeButton->setText("Wait for sensor, Press to interrupt");
        if(program_active) writfirmsensor->setText("Wait for sensor, Press to interrupt");
        if(save_file_active)record_file->setText("Wait for sensor, Press to interrupt");
        toggle=true;
        }
        else {
            if(read_active) sendReadOrderButton->setText("");
            if(write_active) writeButton->setText("");
            if(program_active) writfirmsensor->setText("");
            if(save_file_active)record_file->setText("");
            toggle=false;
        }
    }
}

void AdvanceWirellessSensor::SendAction(){
    flag_timeout=false;
    bool rx_ok=false;
    timer2->start(120000);
    add_sensor_to_list(sensor.serialNb);
    do{
       rx_ok=serialCom->readframe_wmodule(sensor.serialNb);
       QApplication::processEvents();
    }while(!rx_ok&&(read_active||write_active||program_active||save_file_active)&&!flag_timeout);
    if(rx_ok){
        if(save_file_active) save_to_file();
        else send_param();
    }
    timer2->stop();
}

void AdvanceWirellessSensor::save_to_file(){
    uint8_t param[17];
    bool rx_ok=false;
  uint16_t address=0;
    for (int i=0;i<17;i++) param[i]=0;

    uint16_t n_size=0;
    QTextStream  sav_fileStream;
    sav_file->flush();   //erase file
    sav_fileStream.setDevice(sav_file);   //set target for stream
    uint16_t length=8192;
    do
    {
    if(select_memory->currentIndex()==0)//e2p
    {
        param[0]=0;
        param[1]=0x12;
        param[2]=(address>>8)&0xFF;
        param[3]=address&0xFF;
    }
    if(select_memory->currentIndex()==1)//ram
    {
        param[0]=0;
        param[1]=0x14;
        param[2]=(address>>8)&0xFF;
        param[3]=address&0xFF;

    }
    waitFor(5);
            serialCom->sendframe_wdata(sensor.serialNb,0x0F,param,sensor.relay_list[sensor.i_relay].serialNb); //si commande  envoyer
            uint8_t n_try=0;
            do{
                rx_ok=serialCom->readframe_wmodule(sensor.serialNb);
            }while (!rx_ok&&(n_try++<10));
            if(rx_ok){
                QString received_data="";
                for(int i=0;i<8;i++) received_data+=QString::number(serialCom->getAllReceivedData()[34+i],16).rightJustified(2,'0')+" ";
                 sav_fileStream<<QString::number(address,16).rightJustified(4,'0')<<"\t"<<received_data<<"\r\n";
            }
         n_size+=8;
         address+=8;
    }while(n_size<=length)    ;
    add_sensor_to_list(0x0000);
     sav_file->close();
     delete sav_file;
     SaveFileClicked();


}

void AdvanceWirellessSensor::send_param(){
    uint8_t param[17];
    bool rx_ok=false;
    bool ok=true;
    for (int i=0;i<17;i++) param[i]=0;

    if(read_active){
        if(select_memory->currentIndex()==0)//e2p
        {
            param[0]=0;
            param[1]=0x12;
            param[2]=((addressLine->text().toUInt(&ok,16))>>8)&0xFF;
            param[3]=(addressLine->text().toUInt(&ok,16))&0xFF;
        }
        if(select_memory->currentIndex()==1)//ram
        {
            param[0]=0;
            param[1]=0x14;
            param[2]=((addressLine->text().toUInt(&ok,16))>>8)&0xFF;
            param[3]=(addressLine->text().toUInt(&ok,16))&0xFF;

        }
    }


    if(write_active){ //write e2p
        param[0]=0;
        param[1]=0x13;
        param[2]=((addressLineWrite->text().toUInt(&ok,16))>>8)&0xFF;
        param[3]=(addressLineWrite->text().toUInt(&ok,16))&0xFF;
        param[4]=(valueToWrite->text().toUInt(&ok,16))&0xFF;
        if(reset_chip->isChecked()) param[5]=1;
        else param[5]=0;
    }

    if(program_active){
        param[0]=0;
        param[1]=0x13;
        param[2]=0x01;
        param[3]=0x01;
        param[4]=0xE5;
        param[5]=1;
    }

    serialCom->sendframe_wdata(sensor.serialNb,0x0F,param,sensor.relay_list[sensor.i_relay].serialNb); //si commande  envoyer
    if(read_active)
    {
        uint8_t n_try=0;
        do{
            rx_ok=serialCom->readframe_wmodule(sensor.serialNb);
        }while (!rx_ok&&(n_try++<10));
        if(rx_ok){
            QString received_data="";
            for(int i=0;i<8;i++) received_data+=QString::number(serialCom->getAllReceivedData()[34+i],16).rightJustified(2,'0')+" ";
            dataReadDisplay->setText(received_data);
        }
        add_sensor_to_list(0x0000);
    }
    if(read_active) ReadClicked();
    if(write_active) WriteClicked();
}

void AdvanceWirellessSensor::program_sensor(){
    QString filename_tmp="";
    if(settingFile->contains("ADV_CONFIG_W/DIR_SREC_FILE")){
       filename_tmp=(settingFile->value("ADV_CONFIG_W/DIR_SREC_FILE").toString());
    }
    QString srec_filename=QFileDialog::getOpenFileName(this,"Choose Filename",filename_tmp, "*.srec");
    if(srec_filename=="")
    {
        ProgramClicked();
        return;
    }
    settingFile->setValue("ADV_CONFIG_W/DIR_SREC_FILE",QVariant(srec_filename));
    QFile  srec_file(srec_filename);
    if(!srec_file.open((QIODevice::ReadOnly | QIODevice::Text))){
        //QMessageBox::critical(this,"Error Opening File","The file can't be opened");
        write_firm_sensor_status->setText("The file can't be opened");
        ProgramClicked();
        return;
    }
    QByteArray buffer_envoi;
    buffer_envoi.clear();
    uint inc=0;
    vector<QByteArray> lines;
    while (!srec_file.atEnd()) {
        lines.push_back(srec_file.readLine());
    }
    //qDebug()<<"file read ok";
    do{
        QByteArray line = lines.at(inc++);
        buffer_envoi.push_back(line.at(0));
        buffer_envoi.push_back(line.at(1));
        for(int j=2; j<(line.size()-1);j+=2)
        {
            char const tmp[3] = { line.at(j), line.at(j+1), 0 };
            buffer_envoi.push_back((uint8_t)strtol(tmp, 0, 16));
        }
    }
    while (inc<lines.size()) ;
    //qDebug()<<"file read ok";
    lines.clear();

    int32_t i=0;
    i=0;
    QByteArray data_to_send;
    QString dline="";
    data_to_send.clear();

    data_to_send.push_back(0x01); //CMD
    data_to_send.push_back(0x01); //ACK
    data_to_send.push_back(hi(sensor.serialNb)); //adresse MSB
    data_to_send.push_back(lo(sensor.serialNb)); //adresse LSB
    data_to_send.push_back(0x01);
    data_to_send.push_back(0xFF);
    if(!serialCom->send_sensor_firmware(data_to_send)) SendAction();
    if(!program_active) {
        write_firm_sensor_status->setText("Sensor update canceled by user");
        return;
    }
    int n_packet=0;
    uint32_t size_file=buffer_envoi.size();
    int k=0;
    waitFor(200);
    do{
        dline=QString::number(n_packet++)+" : ";
        data_to_send.clear();
        data_to_send.push_back(0x01); //CMD
        data_to_send.push_back(0x01); //ACK
        data_to_send.push_back(hi(sensor.serialNb)); //adresse MSB
        data_to_send.push_back(lo(sensor.serialNb)); //adresse LSB
        data_to_send.push_back(0x01); //
        int j=0;
        do{
            if(k++>527){
                data_to_send.push_back(0xFF);//fill frame with 0xFF
            }
            else
            {
                if(i<buffer_envoi.size())data_to_send.push_back(buffer_envoi.at(i++));
                else data_to_send.push_back(0xFF);//fill frame with 0xFF
            }
        }while(j++<106);
        if(k>527) k=0;

        waitFor(10);
        //  for(int id=0;id<data_to_send.size();id++) dline+=QString::number(uint8_t(data_to_send.at(id)),16).rightJustified(2,'0')+" ";
        //  qDebug()<<dline;

        if(!serialCom->send_sensor_firmware(data_to_send)){
            qDebug()<<"write firm error";
            write_firm_sensor_status->setText("Error when updating sensor");
            ProgramClicked();
            return;
        }

        write_firm_sensor_status->setText("Uploading new firmware : "+QString::number((100*i)/size_file)+" %");
      //  for(int id=0;id<data_to_send.size();id++) dline+=QString::number(uint8_t(data_to_send.at(id)),16).rightJustified(2,'0')+" ";
       // qDebug()<<dline;
    }while((i<buffer_envoi.size())&&program_active);
    if(!program_active) {
        write_firm_sensor_status->setText("Sensor update canceled by user");
        return;
    }
    data_to_send.clear();
    data_to_send.push_back(0x01); //CMD
    data_to_send.push_back(0x01); //ACK
    data_to_send.push_back(hi(sensor.serialNb)); //adresse MSB
    data_to_send.push_back(lo(sensor.serialNb)); //adresse LSB
    data_to_send.push_back(0x01); //
    data_to_send.push_back(0x7A); //
    data_to_send.push_back(0xA8); //
    //send size

    data_to_send.push_back(hhh(size_file)); //
    data_to_send.push_back(hh(size_file)); //
    data_to_send.push_back(hi(size_file)); //
    data_to_send.push_back(lo(size_file)); //
    dline="trame size : ";
 //   for(int id=0;id<data_to_send.size();id++) dline+=QString::number(uint8_t(data_to_send.at(id)),16).rightJustified(2,'0')+" ";
  //  qDebug()<<dline;
    if(!serialCom->send_sensor_firmware(data_to_send)){
        write_firm_sensor_status->setText("Error when updating sensor");
        ProgramClicked();
        return;
    }
    flag_timeout=false;
    bool rx_ok=false;
    timer2->start(10000);
    add_sensor_to_list(sensor.serialNb);
    do{
       rx_ok=serialCom->readframe_wmodule(sensor.serialNb);
       QApplication::processEvents();
    }while(!rx_ok&&!flag_timeout);
    timer2->stop();
    if(rx_ok) write_firm_sensor_status->setText("Sensor successfully updated");
    else write_firm_sensor_status->setText("Error when updating sensor");
    ProgramClicked();
    return;
}


bool AdvanceWirellessSensor::add_sensor_to_list(uint16_t _sensor_nr){
    QByteArray _data_to_send;
    _data_to_send[0]=0x20;
    _data_to_send[1]=_sensor_nr>>8;
    _data_to_send[2]=_sensor_nr&0x00FF;
    waitForUs(100);
    return serialCom->write_serial_data(_data_to_send,0);
}

void AdvanceWirellessSensor::closeEvent(QCloseEvent *event)
 {  //event when window is closed
     flag_timeout=true;
     if(read_active) ReadClicked();
     if(write_active) WriteClicked();
     if(program_active) ProgramClicked();
     if(save_file_active) SaveFileClicked();
     QApplication::processEvents();
     waitFor(200);
     settingFile->setValue("WINDOW_POS/ADV_W_CONFIG_WIN_X",QVariant(QString::number(this->pos().x())));
     settingFile->setValue("WINDOW_POS/ADV_W_CONFIG_WIN_Y",QVariant(QString::number(this->pos().y())));
     setVisible(false);
     delete settingFile;
     emit AdvanceClosed();
 }
