#include "dataacquisition.h"


//*****************************
// Constructor and destructor
//*****************************

DataAcquisition::DataAcquisition(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, vector<Module> _moduleList, QWidget *parent) :
    QWidget(parent)
{   //Constructor
    tsvFile=NULL;
    serialCom = _serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    moduleList=_moduleList;

    fillWindow();
    timerSlot();


    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval();
}

DataAcquisition::~DataAcquisition()
{   //Destructor
    //TODO : delete
}

//*****************************
// GUI
//*****************************
void DataAcquisition::fillWindow()
{   //place GUI elements
    globalLayout = new QVBoxLayout;
    groupBoxData=new QGroupBox("Data Acquisition");
    verticalLayout = new QVBoxLayout;
        gridLayout = new QGridLayout;

            gridLayout->setRowStretch(0,1);
            QLabel *nameLabel= new QLabel("Name :");
            gridLayout->addWidget(nameLabel,1,0,Qt::AlignRight);
            centralNameEdit = new QLineEdit();
            gridLayout->addWidget(centralNameEdit,1,1);

            //Refresh central name :
            centralNameEdit->setText(serialCom->getCentralName());
            centralNameEdit->setEnabled(false);
            //connect(centralNameEdit,SIGNAL(returnPressed()),this,SLOT(centralNameChanged()));

            QLabel *serialNumberLabel= new QLabel("Serial Number :");
            gridLayout->addWidget(serialNumberLabel,2,0,Qt::AlignRight);
            serialNumber = new QLabel(QString::number(serialCom->getCentralSN(),16).toUpper().rightJustified(8,'0'));
            gridLayout->addWidget(serialNumber,2,1);

            gridLayout->setRowStretch(3,1); //space

        //Period of measures
            QLabel *periodLabel= new QLabel("Period :");
            gridLayout->addWidget(periodLabel,4,0,Qt::AlignRight);
            periodValue = new QComboBox;
            QStringList listOfPeriods;
            listOfPeriods <<"1sec"<<"2sec"<<"5sec"<<"10sec"<<"15sec"<<"20sec"<<"30sec"
                    <<"1min"<<"2min"<<"5min"<<"10min"<<"15min"<<"1hour";
            periodValue->addItems(listOfPeriods);
            periodValue->setMaxVisibleItems(20); //>13, 10 by default
            periodValue->setCurrentIndex(-1);
            gridLayout->addWidget(periodValue,4,1);

            connect(periodValue,SIGNAL(currentIndexChanged(int)),this,SLOT(setPeriod(int)));

            gridLayout->setRowStretch(5,1); //space

        //Display date / synchronise
            centralDate = new QLabel;
            centralDate->setText("xx/xx/20xx xx:xx:xx");
            gridLayout->addWidget(centralDate,6,0);
            setTimeButton = new QPushButton("Set Time");
            connect(setTimeButton,SIGNAL(clicked()),this,SLOT(setCentralTime()));
            gridLayout->addWidget(setTimeButton,6,1);

            gridLayout->setRowStretch(7,1);



        //Memory space
            QLabel *nbOfDataLabel= new QLabel("Nb of data :");
            gridLayout->addWidget(nbOfDataLabel,9,0,Qt::AlignRight);
            nbOfData = new QLabel;
            nbOfData->setText("[xxx]");
            gridLayout->addWidget(nbOfData,9,1);

            QLabel *freeMemoryLabel= new QLabel("Free memory :");
            gridLayout->addWidget(freeMemoryLabel,10,0,Qt::AlignRight);
            freeMemorySpace = new QLabel;
            freeMemorySpace->setText("[xxx]");
            gridLayout->addWidget(freeMemorySpace,10,1);

            gridLayout->setRowStretch(11,1);

        verticalLayout->addLayout(gridLayout);
        buttonLayout = new QHBoxLayout;

        //Buttons (back, clear, Download)
            returnButton = new QPushButton("<= Back");
            buttonLayout->addWidget(returnButton);
            connect(returnButton,SIGNAL(clicked()),this,SLOT(backToManageModule()));

            clearMemoryButton = new QPushButton("Clear Memory");
            buttonLayout->addWidget(clearMemoryButton);
            connect(clearMemoryButton,SIGNAL(clicked()),this,SLOT(clearMemory()));

            downloadButton = new QPushButton("Download");
            downloadButton->setDisabled(true);
            buttonLayout->addWidget(downloadButton);
            connect(downloadButton,SIGNAL(clicked()),this,SLOT(downloadDataClicked()));

        verticalLayout->addLayout(buttonLayout);
    groupBoxData->setLayout(verticalLayout);
    globalLayout->addWidget(groupBoxData);
    setLayout(globalLayout);
}

bool DataAcquisition::writedata(uint32_t serialNb,uint32_t address,uint8_t datatowrite){

    //serialCom->deleteErrorCom();
    bool rx_ok=false;
    int n_try=3;
    do{
        waitFor(10);
        if(serialCom->writeByteAtWithRetry(serialNb, address, datatowrite,1)) break;
        rx_ok=serialCom->readBytesAtWithRetry(serialNb, address,1,3);
        if(rx_ok&&(serialCom->getReceivedData()[0]!=datatowrite)) rx_ok=false;
        if(!rx_ok&&!n_try)
        {
            int response=QMessageBox::warning(0,"Communication failure","Cannot reach the system and write data.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
            if(response==QMessageBox::Cancel)
            {
                return false;
            }
            else n_try=3;
        }
    }while(!rx_ok);
    if(!rx_ok) return false;
    return true;
}

//*****************
//  Button Slots
//*****************

void DataAcquisition::setPeriod(int indexChanged)
{   //Change period of measure

    uchar valueToChange=uchar(indexChanged);

    // for old central, all values are not available
    if((indexChanged==8||indexChanged==9||indexChanged==11)&&(serialCom->getCentralSN()<0x01000400))
    {
        QMessageBox::warning(this, "Warning : Incompatible value", "Warning : This period value is not compatible with your central version");
    }
    else
    {
        if(indexChanged>7)  //to change index to value to send
        {                   // historically in Palm soft, delays were not placed in order
            if(indexChanged==8) valueToChange=10;
            if(indexChanged==9) valueToChange=11;
            if(indexChanged==10) valueToChange=8;
            if(indexChanged==11) valueToChange=12;
            if(indexChanged==12) valueToChange=9;
        }
        else valueToChange=uchar(indexChanged);
    }
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000013,valueToChange,3);
}



void DataAcquisition::centralNameChanged()
{   //send edited central name

    if(!serialCom->setModuleName(serialCom->getCentralSN(),0x01000020,centralNameEdit->text()))
    {
        QMessageBox::critical(this,"new name not writed","Name were not writed, please retry.",QMessageBox::Ok);
    }
    else centralNameEdit->setText(serialCom->getCentralName());
}

void DataAcquisition::clearMemory()
{   //place pointer to 0 to "erase" campaign

    commonRefreshTimer->stop();

    int clearingResponse = QMessageBox::question(this, "Clear memory", "The memory will be competely erased, are you sure ?", QMessageBox::Yes | QMessageBox::No);
    if (clearingResponse == QMessageBox::Yes)
    {
        // Clear overflow
        bool ok=serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000014,0x00,3);
        if(!ok) qDebug()<<"error clearing overflow";

          // Clear address pointer
        for(int i=0;i<3;i++)
        {
            ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000010+uint32(i),0x00,3);
        }
        if(!ok) qDebug()<<"error clearing address pointer";
        // Writing at 0x12 makes the concentrator store a first record. (pointer change immediately)
        if(ok)
        {
            QMessageBox::information(this, "Clearing successful", "Campaign has been cleared !");
        }
    }
    timerSlot();
}

void DataAcquisition::backToManageModule()
{   //button back clicked, stop window and return to module grid
    commonRefreshTimer->disconnectTimeout();
    emit backToMainWindow();    //Signal to restart the main window (refresh Values)
}

void DataAcquisition::startTimerAgain()
{
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

void DataAcquisition::setCentralTime()
{

    //send local clock time to central
   /* writedata(serialCom->getCentralSN(),0x123,bcdFromDecimal(QTime::currentTime().hour()));
    writedata(serialCom->getCentralSN(),0x124,bcdFromDecimal(QTime::currentTime().minute()));
    writedata(serialCom->getCentralSN(),0x125,bcdFromDecimal(QTime::currentTime().second()));
    writedata(serialCom->getCentralSN(),0x120,bcdFromDecimal(QDate::currentDate().year()%100));
    writedata(serialCom->getCentralSN(),0x121,bcdFromDecimal(QDate::currentDate().month()));
    writedata(serialCom->getCentralSN(),0x122,bcdFromDecimal(QDate::currentDate().day()));*/

    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x123,bcdFromDecimal(QTime::currentTime().hour()),5);
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x124,bcdFromDecimal(QTime::currentTime().minute()),5);
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x125,bcdFromDecimal(QTime::currentTime().second()),5);
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x120,bcdFromDecimal(QDate::currentDate().year()%100),5);
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x121,bcdFromDecimal(QDate::currentDate().month()),5);
    serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x122,bcdFromDecimal(QDate::currentDate().day()),5);
}

void DataAcquisition::downloadDataClicked()
{

    //Download campaign asked
    commonRefreshTimer->stop();

    bool ok=true;
    // update memory pointer, to be sure using latest values
    if(!updateMemoryInfo()) QMessageBox::warning(this,"Error updating pointer info",
                "Memory info is not updated, retry ?",QMessageBox::Retry,QMessageBox::Cancel); //TODO : button actions

    //Count nb of lines (wireless sensors)
    int nbOfWirelessModules=0;
    int totalNbOfLines=0;
    for(uint moduleIndex=0;moduleIndex<moduleList.size(); moduleIndex++)
    {
        if(moduleList[moduleIndex].typeOfModule==wirelessModule)
        {
            nbOfWirelessModules++;
            totalNbOfLines+=moduleList[moduleIndex].nbOfLines;
        }
    }


    //MessageBox before launching download
    QString messageString("You will download the current campaign :<UL> <LI> ");
    messageString+=QString::number(moduleList.size())+" modules";
    if(nbOfWirelessModules>0)
        messageString+=" ("+QString::number(nbOfWirelessModules)+" wireless - "+QString::number(6*totalNbOfLines)+" sensors)";
    messageString+=" + 1 central.<LI> ";
    if(memoryOverflow>0) messageString.append(QString::number(endOfMemory/12)+" records.</UL> ");
    else  messageString.append(QString::number(memoryPointer/12)+" records.</UL> ");

    int clearingResponse = QMessageBox::question(this, "Download Campaign Data", messageString, QMessageBox::Ok, QMessageBox::Cancel);
    if (clearingResponse == QMessageBox::Ok)
    {
        //init table of downloadedValues
        tableOfDownloadedValues.clear();
        vector <uchar> emptyTable;
        if(memoryOverflow>0) emptyTable.reserve(endOfMemory);   //reserve size
        else emptyTable.reserve(memoryPointer);
        tableOfDownloadedValues.resize(moduleList.size()+1,emptyTable);  //place empty

        //reserve more memory for wireless
        for(uint32 moduleIndex=0; moduleIndex<(moduleList.size()); moduleIndex++)
        {
            if(moduleList[moduleIndex].typeOfModule==wirelessModule)
            {
                if(memoryOverflow>0) tableOfDownloadedValues[moduleIndex+1].reserve(endOfMemory*moduleList[moduleIndex].nbOfLines);   //reserve size
                else tableOfDownloadedValues[moduleIndex+1].reserve(memoryPointer*moduleList[moduleIndex].nbOfLines);
            }
        }

        //Start to download data
        ok=downloadCampaign();
       // debugDisplayTable();//---------------------------------------------
        //Then generate the file from downloadTableValues
        if(ok) ok=generateFile();
        else QMessageBox::critical(this,"Download aborted","Campaign download did not succeed");
    }

    //Restart refresh
    timerSlot();
}

//*****************************
// Update/Refresh window info
//*****************************

void DataAcquisition::timerSlot()
{   // connected to refresh timer, update values in the window

    commonRefreshTimer->stop();

    //serialCom->deleteErrorCom();    //retry

    //refresh date and time :
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x120,1,3);
    if(!ok)
        centralDate->setText("xx/xx/20xx xx:xx:xx");
    else
    {
        QString dateToShow = QString::number(serialCom->getReceivedData()[2],16).rightJustified(2,'0')+"/"   //day
                +QString::number(serialCom->getReceivedData()[1],16).rightJustified(2,'0')+"/20"      //month
                +QString::number(serialCom->getReceivedData()[0],16).rightJustified(2,'0')+" "        //year
                +QString::number(serialCom->getReceivedData()[3],16).rightJustified(2,'0')+":"        //hour
                +QString::number(serialCom->getReceivedData()[4],16).rightJustified(2,'0')+":"        //min
                +QString::number(serialCom->getReceivedData()[5],16).rightJustified(2,'0');        //sec
        centralDate->setText(dateToShow);
    }

    //refresh Nb of Data and free memory space :
    if(!updateMemoryInfo())
    {
        nbOfData->setText("[xxx]");
        freeMemorySpace->setText("[xxx]");
    }
    else
    {
        // nb of data & free memory :
        if(memoryOverflow>0)
        {
            nbOfData->setText(QString::number(endOfMemory/12)+" records");
            freeMemorySpace->setText("0% (Rollover)");
        }
        else
        {
            nbOfData->setText(QString::number(memoryPointer/12)+" records");
            double freeMemory=((endOfMemory-memoryPointer)*100 /endOfMemory);
            freeMemorySpace->setText(QString::number(freeMemory)+"%");
        }
        downloadButton->setEnabled(true);

        //refresh period value :
        periodValue->disconnect();
        if(serialCom->getReceivedData()[3]>7)
        {
            if(serialCom->getReceivedData()[3]==8) periodValue->setCurrentIndex(10);
            if(serialCom->getReceivedData()[3]==9) periodValue->setCurrentIndex(12);
            if(serialCom->getReceivedData()[3]==10) periodValue->setCurrentIndex(8);
            if(serialCom->getReceivedData()[3]==11) periodValue->setCurrentIndex(9);
            if(serialCom->getReceivedData()[3]==12) periodValue->setCurrentIndex(11);
        }
        else periodValue->setCurrentIndex(int(serialCom->getReceivedData()[3]));
        connect(periodValue,SIGNAL(currentIndexChanged(int)),this,SLOT(setPeriod(int)));
    }
    commonRefreshTimer->startInterval();    //start for next update
}

bool DataAcquisition::updateMemoryInfo()
{   //get info to update memoryPointer & memoryOverflow
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000010,1,3);
    if(!ok)
        qDebug()<<"error on quering free space value";
    else
    {
        memoryPointer=serialCom->getReceivedData()[0]   //24bits, bigEndian
                +uint32(serialCom->getReceivedData()[1]<<8)
                +uint32(serialCom->getReceivedData()[2]<<16);
        endOfMemory = 0x3C000;   // End of External EEPROM
        if( ((serialCom->getReceivedData()[6]==0x01)||(serialCom->getReceivedData()[6]==0x02))&&(serialCom->getReceivedData()[7]!=0xFF)) {
            //memoryconfig->setEnabled(true);
            typeofcentral=1;
        }
        else
        {
            typeofcentral=0;
           // memoryconfig->setEnabled(false);
        }
        if(serialCom->getReceivedData()[6]==0x02 && serialCom->getReceivedData()[7]==0x01)
        {
        //    memoryconfig->setCurrentIndex(1);
            endOfMemory=0x7E004; // End of External EEPROM Central 2 (double memory)
        }
        //else memoryconfig->setCurrentIndex(0);
        memoryOverflow=serialCom->getReceivedData()[4];
    }
    return ok;
}

//***********************
//  Saving data methods
//***********************

bool DataAcquisition::downloadCampaign()
{

    updateMemoryInfo();
    //Init the progress dialog
    QProgressDialog *downloadProgress = new QProgressDialog("Data transfer in progress...", "Cancel", 0, 100);
    downloadProgress->setWindowTitle("Downloading campaign...");
    downloadProgress->setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/MVicon.png"));
    downloadProgress->setMinimum(0);    //%
    downloadProgress->setMaximum(100);
    downloadProgress->setWindowModality(Qt::ApplicationModal);
    downloadProgress->show();
    downloadProgress->setValue(0);

    //Useful variables :
    uint32 address=0;
    uint32 maximumAddress=1; //set later
    uint32 currentSN=0;
    int maxLineIndex=1; //default central
    if(serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000016,1,3)){
        qDebug()<<"dble memoire : "<<serialCom->getReceivedData()[0];
        if( ((serialCom->getReceivedData()[0]==0x01)||(serialCom->getReceivedData()[0]==0x02))&&(serialCom->getReceivedData()[0]!=0xFF)) typeofcentral=1;
        else typeofcentral=0;
    }
    else
    {
        return false;  //can't get central type
        qDebug()<<"can't get central type";
    }
    qDebug()<<"central type : "<<typeofcentral;


    // maximum Address :
    //In case of overflow, the full memory is read, unwanted or overwrited samples are deleted after
    if(memoryOverflow==0) maximumAddress=memoryPointer;
    else maximumAddress=endOfMemory;
    qDebug()<<"maw address"<<QString::number(maximumAddress,16);
    //Compute total nb of transfer (for progress)
    uint totalNbOfLines=1; //start with central = 1 line
    for(uint i=0; i<moduleList.size(); i++)
    {
        totalNbOfLines+=moduleList[i].nbOfLines;
        if(moduleList[i].typeOfModule==powerModule2) totalNbOfLines++;
    }
    uint nbOfDownloadPerLines = ceil(maximumAddress/64.0);
    uint totalNbOfDownload = totalNbOfLines*nbOfDownloadPerLines;
    int downloadNb=0;   //index of download (for progress percent)
    //Be careful : computed for progress, not the exact nb of DL
    //(eg : lines for wirelessmodules are adjacent => less number of reading in reality)

//******************************
// Fetch data from each module

    // This loop will process all modules + central, one by one
    for(uint32 moduleIndex=0; moduleIndex<(moduleList.size()+1); moduleIndex++)
    {
        // update currentSN from list
        if(moduleIndex==0)  // central
            currentSN=serialCom->getCentralSN();

        else    //other modules
            currentSN=moduleList.at(moduleIndex-1).serialNb;

        qDebug()<<"> module nb "<<moduleIndex<<", current SN : "<<QString::number(currentSN,16);

//****************************
// Fetching names for header

        //fetching channel name
        if(moduleIndex>0)   //all modules (except central)
        {
            bool ok=true;
            //Reading channel names
            if(moduleList[moduleIndex-1].typeOfModule==wirelessModule)
            {
                //get all information about sensors (SN, name, type, coeff...)
                if(!getWirelessSensorList(moduleIndex-1))
                {
                    qDebug()<<"error get Sensor List";
                    return false;
                }
            }
            else    //not wireless
            {
                //reading names :
                uchar channelNames[96]; //6 names of 16 bytes

                //32 bytes (2 names of 16 bytes)
                ok = ok & serialCom->readBytesAtWithRetry(currentSN,0x010000E0,4,5);
                if(ok) for(int i=0; i<32; i++) channelNames[i+64]=serialCom->getReceivedData()[i];
                //64 bytes (4 names of 16 bytes)
               // for(int i=0;i<6;i++)
                ok=ok & serialCom->readBytesAtWithRetry(currentSN,0x010000A0,8,5);
                if(ok) for(int i=0; i<64; i++) channelNames[i]=serialCom->getReceivedData()[i];


                // fetching channel names for channelNames tab
                if(ok)  //DL success
                {
                    for(int channel=0;channel<6;channel++)
                    {
                        char channelName[17];   //16 char + 0x00
                        for(int charPosition=0; charPosition<16; charPosition++)
                        {   //copy characters
                            if((channelNames[channel*16+charPosition]!=0x00) && (channelNames[channel*16+charPosition]!=0xFF))
                                channelName[charPosition]=channelNames[channel*16+charPosition];
                            else
                            {
                                channelName[charPosition]='\0';
                                break;
                            }
                        }
                        moduleList[moduleIndex-1].channelName[channel]=QString::fromStdString(channelName);
                    }
                }
                else
                {
                    qDebug()<<"Error downloading, cannot continue";
                    return false;  //DL error : cannot continue
                }
            }
            if(!ok) qDebug()<<"failed in channel name reading";
        }

//****************************
// Fetching campaign data
        int old_percent=0;
        if(moduleIndex<1) maxLineIndex=1;   // central
        else
        {
            maxLineIndex=moduleList.at(moduleIndex-1).nbOfLines;   // other modules
            if(moduleList[moduleIndex-1].typeOfModule==powerModule2) maxLineIndex=2;
        }

        for(int lineIndex=0; lineIndex<maxLineIndex; lineIndex++)   //each line
        {

           /* if((moduleList[moduleIndex-1].typeOfModule==powerModule2)&&(lineIndex==1))maximumAddress=(endOfMemory&0x00FFFFFF)|0x04000000;
                    else maximumAddress=endOfMemory;*/
            qDebug()<<"maximum address"<<QString::number(maximumAddress,16).rightJustified(8,'0').toUpper();
            // address loop
            for(uint32 indexOfTransfer=0; indexOfTransfer<maximumAddress; indexOfTransfer+=64)  //64 bytes transfers
            {
                address=indexOfTransfer;

                if(moduleIndex>0)
                {
                        if(moduleList[moduleIndex-1].typeOfModule==powerModule2)
                        {
                            // limit address to the max of memory
                            address &= 0x07FFFF;
                        }
                        else
                        {
                        // limit address to the max of memory
                        if(moduleList.at(moduleIndex-1).memoryAvailable==2) address &= 0x07FFFF;
                        else address &= 0x03FFFF;

                        //add line number in the address
                        address = address | (uint32(lineIndex)<<18);
                        }

                    if((moduleList[moduleIndex-1].typeOfModule==powerModule)&&(moduleIndex>=1)&&(typeofcentral==1)) //pas wireless , pas concentrateur,  sur central2
                    {
                           address|=0x03800000;    //add 0380 0000 for the address offset
                    }
                    else
                    {
                        if((moduleList[moduleIndex-1].typeOfModule==powerModule2)&&(lineIndex==1))address|=0x04000000;  //to get reactive power
                        else
                        address|=0x03000000;    //add 0300 0000 for the address offset
                    }
                }
                else address|=0x03000000;
                //Reading values (64 uchar) :
                int n_try=3;
                bool ok=false;
                qDebug()<<"adress :"<<address;
                do
                {
                    ok=serialCom->readBytesAtWithRetry(currentSN,address,8,5);
                    if(serialCom->getReceivedDataLength()<64) ok=false;
                    if (!ok){
                        waitFor(100);
                        //serialCom->deleteErrorCom();
                    }
                }while(!ok&&n_try--);
                int response = QMessageBox::Retry;
                while(!ok && response==QMessageBox::Retry)
                {
                    // In case of reading error, retry with a message window
                    response=QMessageBox::warning(this,"Error during download","Reading address 0x"+QString::number(address,16)+", from Module SN "+QString::number(currentSN,16).rightJustified(8,'0')+", failed.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
                    if(response==QMessageBox::Retry)
                    {
                        //serialCom->deleteErrorCom();
                        n_try=3;
                        do
                        {
                            ok=serialCom->readBytesAtWithRetry(currentSN,address,8,5);
                            if(serialCom->getReceivedDataLength()<64) ok=false;
                            if (!ok){
                                waitFor(100);
                                //serialCom->deleteErrorCom();
                            }
                        }while(!ok&&n_try--);
                    }
                    else
                    {
                        downloadProgress->close();
                        delete downloadProgress;
                        return false;
                    }
                }

                if(ok)
                {
                    downloadProgress->activateWindow();
                    downloadNb++; //for progress display
                    //Write values in the table
                    if(tableOfDownloadedValues.size()>moduleIndex)
                    {
                        //QString dline="";
                        for(int tableIndex=0;tableIndex<64;tableIndex++)
                        {
                                tableOfDownloadedValues[moduleIndex].push_back(serialCom->getReceivedData()[tableIndex]);
                               // dline+=QString::number(serialCom->getReceivedData()[tableIndex],16).rightJustified(2,'0')+" ";
                            //Be careful : for next line, the adress will be a 64 multiple, and not necessarily a 12 multiple !
                        }
                      //  qDebug()<<address<<" : "<<dline;
                    }
                    else    //out of bounds
                    {
                        QString messageModule="Module line "+QString::number(moduleIndex)+"can't be accessed in the table (max="
                                +QString::number(tableOfDownloadedValues.size())+")";
                        QMessageBox::critical(this,"Invalid Access",messageModule);
                        return false;       // Should never happened //TODO : delete ?
                    }

                // compute progress for QProgress display

                    int percent = (100 * float(downloadNb))/totalNbOfDownload;
                    if(percent>100) percent=100;
                    if(percent!=old_percent)
                    {
                        downloadProgress->setValue(percent);
                        old_percent=percent;
                    }
                    if(downloadProgress->wasCanceled())
                    {
                        downloadProgress->close();
                        delete downloadProgress;
                        return false;
                    }
                    QCoreApplication::processEvents();

                }// end of read data computing (return of readBytes)

            }   //end of address loop

        }   //end of lineIndex loop

    }   // end of moduleSN loop

    //end of download
    downloadProgress->setValue(100);

//****************************
//  Overflow/offset managing

    //If there is an overflow, an overwriting of the campaign data may occur during download
    // pointers are stored before and after download, in order to check overwriting
    memoryPointerBeforeDownload=memoryPointer;
    if(updateMemoryInfo())
    {
        memoryPointerAfterDownload=memoryPointer;
    }
    else QMessageBox::critical(this,"Error updating pointer info", "Memory info is not updated, old values can appear in data file",QMessageBox::Cancel);

    delete downloadProgress;
    return true;
}

bool DataAcquisition::getWirelessSensorList(int _wirelessModuleIndex)
{   // Fill the list of sensors in the module structure
    moduleList[_wirelessModuleIndex].sensorList.clear();

    //Nb of reading of 64 requested to fetch sensor names
    int nbReadingNames=ceil(moduleList[_wirelessModuleIndex].nbOfLines*3.0/2.0);
    uchar downloadedValuesSerialNb[96]; //48*2bytes
    uchar downloadedValuesChannelType[64];  //8lines*8
    uchar downloadedValuesSensorNames[nbReadingNames*64];

    bool ok=true;
    //reading sensor SN - SensorSN : 0x010000A0 - little endian
    ok=serialCom->readBytesAtWithRetry(moduleList[_wirelessModuleIndex].serialNb,0x010000A0,8,3);
    if(!ok) return false;
    for(int charIndex=0; charIndex<64; charIndex++) //copy to downloadedValuesSerialNb
        downloadedValuesSerialNb[charIndex]=serialCom->getReceivedData()[charIndex];

    if(moduleList[_wirelessModuleIndex].nbOfLines>5)    // 2nd reading of 32 bytes requested
    {
        ok=serialCom->readBytesAtWithRetry(moduleList[_wirelessModuleIndex].serialNb,0x010000A0+64,4,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<32; charIndex++)
            downloadedValuesSerialNb[charIndex+64]=serialCom->getReceivedData()[charIndex];
    }

    //reading sensor Channel - SensorCh : 0x01000040.
    int nbReadingChannel=8;
    if(moduleList[_wirelessModuleIndex].nbOfLines<=5) nbReadingChannel=4;
    ok=serialCom->readBytesAtWithRetry(moduleList[_wirelessModuleIndex].serialNb,0x01000040,nbReadingChannel,3);
    if(!ok) return false;
    for(int charIndex=0; charIndex<nbReadingChannel*8; charIndex++)//Place in the table :
        downloadedValuesChannelType[charIndex]=serialCom->getReceivedData()[charIndex];

    //reading Name - 0x031FFD00 + 16 * sensorIndex
    for(int readingIndex=0; readingIndex<nbReadingNames; readingIndex++)
    {
//        qDebug()<<"reading Names @"<<QString::number(0x031FFD00+readingIndex*64,16);
        ok=serialCom->readBytesAtWithRetry(moduleList[_wirelessModuleIndex].serialNb,0x031FFD00+readingIndex*64,8,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<64; charIndex++)//Place in the table :
            downloadedValuesSensorNames[charIndex+readingIndex*64]=serialCom->getReceivedData()[charIndex];
    }

    //add each sensor :
    for(uchar sensorIndex=0; sensorIndex<moduleList[_wirelessModuleIndex].nbOfLines*6; sensorIndex++)
    {
        Sensor currentSensor;
        //read Serial Nb
        currentSensor.serialNb=peekUint16FromUcharTabLittleEndian(downloadedValuesSerialNb,sensorIndex*2);
        currentSensor.channelType=downloadedValuesChannelType[sensorIndex];


        if(currentSensor.serialNb>0x1499 && currentSensor.serialNb<0x2000)
        {//Sensor Type delayana
            currentSensor.channelType2=4;
        }
        else currentSensor.channelType2=0;

        char sensorName[17];
        for(uint32 i =0; i<16;i++)
        {
            if(downloadedValuesSensorNames[i+sensorIndex*16]!=(0x00)&&downloadedValuesSensorNames[i+sensorIndex*16]!=(0xFF))
                sensorName[i]=downloadedValuesSensorNames[i+sensorIndex*16];
            else {
                sensorName[i]='\0'; //clear next char
                break;
            }
        }
        sensorName[16]='\0';
        currentSensor.sensorName=QString(sensorName);

        //select unit, coeff and precision from type
        switch(currentSensor.channelType)
            {
                case 0://Temperature7  
                    currentSensor.coefficient=0.01;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="C";
                    break;
                case 1://Humidity
                    currentSensor.coefficient=.01;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="%RH";
                    break;
                case 2://Volts
                    currentSensor.coefficient=.01;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="V";
                    break;
                case 3://Amps
                    currentSensor.coefficient=.01;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="A";
                    break;
                case 4://Active Power
                    currentSensor.coefficient=.1;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="W";
                    break;
                case 5://Apparent Power
                    currentSensor.coefficient=.1;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="VA";
                    break;
                case 6://Light
                    currentSensor.coefficient=100.0/255.0;
                    currentSensor.displayPrecision=0;
                    currentSensor.unitString="%";
                    break;
                case 7://Average Temp
                    switch(currentSensor.channelType2)
                    {
                    case 4:
                        currentSensor.coefficient=1;
                        currentSensor.displayPrecision=0;
                        currentSensor.unitString="";
                        break;
                    default:
                        currentSensor.coefficient=.01;
                        currentSensor.displayPrecision=1;
                        currentSensor.unitString="C";
                        break;
                    }
                    break;
                 case 9:
                        currentSensor.coefficient=.016;
                        currentSensor.displayPrecision=2;
                        currentSensor.unitString="Pa";
                        break;
                 case 8:
                        currentSensor.coefficient=1;
                        currentSensor.displayPrecision=0;
                        currentSensor.unitString="lux";
                        break;
                 case 0xA:
                        currentSensor.coefficient=1;
                        currentSensor.displayPrecision=0;
                        currentSensor.unitString="ppm";
                        break;
                 case 0xB:
                        currentSensor.coefficient=1;
                        currentSensor.displayPrecision=0;
                        currentSensor.unitString="";
                        break;
                case 0x80://Pulses
                    currentSensor.coefficient=1;
                    currentSensor.displayPrecision=0;
                    currentSensor.unitString="";
                    break;
                case 0x81://ON Time
                    currentSensor.coefficient=1;
                    currentSensor.displayPrecision=0;
                    currentSensor.unitString="s";
                    break;
                case 0x90://Active Energy
                    currentSensor.coefficient=.1;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="Wh";
                    break;
                case 0x91://App. Energy
                    currentSensor.coefficient=.1;
                    currentSensor.displayPrecision=1;
                    currentSensor.unitString="VAh";
                    break;
                case 0x92://Switch ON (Lamp)
                    currentSensor.coefficient=1;
                    currentSensor.displayPrecision=0;
                    currentSensor.unitString="";
                    break;
                default:        // Error if can't be determined (mainly for debug)
                    currentSensor.coefficient=1;
                    currentSensor.displayPrecision=0;
                    currentSensor.unitString="Err";
        }
        moduleList[_wirelessModuleIndex].sensorList.push_back(currentSensor);
    }
    return true;
}


bool DataAcquisition::generateFile()
{
    //Init the progress dialog
    QProgressDialog *fileGenerationProgress = new QProgressDialog("Data processing...", "Cancel", 0, 100);
    fileGenerationProgress->setWindowTitle("Creation of file...");
    fileGenerationProgress->setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/MVicon.png"));
    fileGenerationProgress->setMinimum(0);
    fileGenerationProgress->setMaximum(100);
    //fileGenerationProgress->setWindowModality(Qt::Window);
    fileGenerationProgress->setValue(0);

    bool ok=true;

//*****************
//    Open file :

    QString pathOfFile="file.tsv";


    // init file for writing
    tsvFile = new QFile(pathOfFile);  //ou dossier par dfaut?
    if(!tsvFile->open(QFile::WriteOnly | QFile::Truncate))
    {
        ok=false;
        int response = QMessageBox::Retry;
        while(response==QMessageBox::Retry)     //in case of tsv file is already open
        {
            QString message="File "+pathOfFile+" cannot be opened, retry ?";
            response= QMessageBox::information(this,"Cannot open file",message,QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
            if(response==QMessageBox::Retry)
                if(tsvFile->open(QFile::WriteOnly | QFile::Truncate))
                {
                    response=0;
                    ok=true;
                }
        }
    }
    tsvFile->flush();   //erase file
    tsvFileStream.setDevice(tsvFile);   //set target for stream

//*********************
//    Header lines

    // Write file header :
    // 1rst line : getCentralSN + 3x Volts + NbModules x(6x serialNb)
    QString headerLine=QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0');
    headerLine+="\tVolts\tVolts\tVolts";

    for(uint32 i=0; i<moduleList.size(); i++)
    {
        uint nbOfChannel=moduleList[i].nbOfLines*6;
        for(uint32 j=0; j<nbOfChannel; j++)
        {
            headerLine+="\t"+QString::number(moduleList[i].serialNb,16).rightJustified(8,'0');
            if(moduleList[i].typeOfModule==powerModule2)
                headerLine+="\t"+QString::number(moduleList[i].serialNb,16).rightJustified(8,'0');
        }
    }

    tsvFileStream<<headerLine;
    headerLine.clear();

    // 2nd line : GetCentralName + Ph 1 + Ph 2 +Ph 3 + NbModules x(Ch1 + Ch2 .. Ch6)
    headerLine=serialCom->getCentralName()+"\tPh 1\tPh 2\tPh 3";

    for(uint16 moduleIndex=0; moduleIndex<moduleList.size(); moduleIndex++)
    {
        for(int channelIndex=0; channelIndex<moduleList[moduleIndex].nbOfLines*6; channelIndex++)  // add channel names
        {
            if(moduleList[moduleIndex].typeOfModule==wirelessModule)
                headerLine+="\t"+moduleList[moduleIndex].sensorList[channelIndex].sensorName;
            else
            {
                headerLine+="\t"+moduleList[moduleIndex].channelName[channelIndex];
                if(moduleList[moduleIndex].typeOfModule==powerModule2)
                    headerLine+="\t"+moduleList[moduleIndex].channelName[channelIndex]+" R";
            }
        }
    }

    tsvFileStream<<"\r\n"<<headerLine;
    headerLine.clear();

//***********************
//  Write samples loop

    //Set the offsetPerLines Array
    for(uint32 moduleIndex=0; moduleIndex<moduleList.size(); moduleIndex++)
    {
        if(moduleList.at(moduleIndex).nbOfLines>0)//si nbOfLines=0 : module wireless sans sensor, donc pas de donnes  rcuprer
        {
            offsetPerLine[moduleIndex] = tableOfDownloadedValues[moduleIndex+1].size()/moduleList.at(moduleIndex).nbOfLines;
        }
    }
    qDebug()<<"process data";
   // uint8_t percent_old=0;
    // overflow, but not during download
    if((memoryOverflow>0) && (memoryPointerBeforeDownload<memoryPointerAfterDownload))
    {
        uint32 firstSample=memoryPointerAfterDownload;
        uint32 endOfFirstLoop=endOfMemory;
        uint32 endOfSecondLoop=memoryPointerBeforeDownload;
        qDebug()<<"first : "<<firstSample<<" endoffirst : "<<endOfFirstLoop<<" end of second : "<<endOfSecondLoop;
        double totalNbOfSamples=endOfFirstLoop-firstSample+endOfSecondLoop;

        //1rst loop (pointer to end)
        qDebug()<<"first loop";
        for(uint32 addressIndex=firstSample; addressIndex<endOfFirstLoop; addressIndex+=12)
        {
            if((addressIndex>=0x3FFF0)&&(addressIndex<0x40000)) addressIndex=0x40000;
            if(addressIndex<0x7E004)
                {
                qDebug()<<"adress"<<addressIndex;
                ok = ok & writeSample(addressIndex);
                // set progress to display
                uint8_t percent = (100 * double(addressIndex-firstSample))/totalNbOfSamples;
                if(percent>100) percent=100;
            //*   if(percent!=percent_old)
             //   {
                //    percent_old=percent
                    fileGenerationProgress->setValue(percent);
                    QCoreApplication::processEvents();
                    if(fileGenerationProgress->wasCanceled())
                    {
                        fileGenerationProgress->close();
                        return false;
                    }
               // }
            }
        }

        //2nd loop (beginning to old pointer)
      //  percent_old=0;
        qDebug()<<"2nd loop";
        for(uint32 addressIndex=0; addressIndex<endOfSecondLoop; addressIndex+=12)
        {
            if((addressIndex>=0x3FFF0)&&(addressIndex<0x40000)) addressIndex=0x40000;
            if(addressIndex<0x7E004)
            {
                 qDebug()<<"adress"<<addressIndex;
                ok = ok & writeSample(addressIndex);
                // set progress to display
                uint8_t percent = (100 * double(endOfFirstLoop-firstSample+addressIndex))/totalNbOfSamples;
                if(percent>100) percent=100;
              //  if(percent!=percent_old)
              //  {
              //      percent_old=percent;
                    fileGenerationProgress->setValue(percent);
                    QCoreApplication::processEvents();
                    if(fileGenerationProgress->wasCanceled())
                    {
                        fileGenerationProgress->close();
                        return false;
                    }
              //  }
            }
        }
    }
    else //no overflow or overflow during download
    {
        uint32 firstSample;
        uint32 maxSampleIndex;

        if(memoryOverflow>0) // Memory overflow during download
        {
            firstSample=memoryPointerAfterDownload;
            maxSampleIndex=memoryPointerBeforeDownload;
        }
        else    //No memory overflow
        {
            firstSample=0;
            maxSampleIndex=memoryPointerBeforeDownload;
        }

        //loop

        for(uint32 addressIndex=firstSample; addressIndex<maxSampleIndex; addressIndex+=12)
        {
            if((addressIndex>=0x3FFF0)&&(addressIndex<0x40000)) addressIndex=0x40000;
            if(addressIndex<0x7E004)
                {
                 qDebug()<<"adress"<<addressIndex;
                ok = ok & writeSample(addressIndex);
                // set progress to display
                uint8_t percent = (100 * (addressIndex-firstSample))/(maxSampleIndex-firstSample);
                if(percent>100) percent=100;
            //    if(percent!=percent_old)
           //     {
            //        percent_old=percent;
                    fileGenerationProgress->setValue(percent);
                    QCoreApplication::processEvents();
                    if(fileGenerationProgress->wasCanceled())
                    {
                        fileGenerationProgress->close();
                        return false;
                    }
             //   }
            }
        }
    }
    //Generation finished
    delete fileGenerationProgress;

//*****************************************
//  Message write OK / Save in other file

    if(ok)
    {
        //Offer to save file
        QString message="The file ["+pathOfFile+"] has been successfully generated.<br>Click on Save to copy in another target.";
        int response =QMessageBox::information(this,"File correctly generated",message,QMessageBox::Ok|QMessageBox::Save,QMessageBox::Ok);
        if(response==QMessageBox::Save)
        {
            //QString pathFile="";
            QDir datafile;
            QDateTime currentDateTime= QDateTime::currentDateTime();
            QSettings *settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
            if(settingFile->contains("MV_DATA/DIR")){
               datafile.setPath(settingFile->value("MV_DATA/DIR").toString());
            }

            QString suggestedName=QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0')+"_"+currentDateTime.toString("ddMMyy_hhmmss")+".tsv";
            QString pathFile=datafile.absolutePath()+"/"+suggestedName;
            QString fileForCopy = QFileDialog::getSaveFileName(this, "Save generated file", pathFile, "Tab Separated Value (*.tsv);;Text file (*.txt)");
            datafile.setPath(fileForCopy);
            QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

            settingFile->setValue("MV_DATA/DIR",QVariant(datafile.absolutePath()));
            delete settingFile;
            if(fileForCopy.size()>0)    // if not Cancel
            {
                bool copySuccess=tsvFile->copy(fileForCopy);
                if(copySuccess)
                {
                    QString message="The file ["+fileForCopy+"] has been successfully saved";
                    QMessageBox::information(this,"File correctly saved",message,QMessageBox::Ok);
                }
                else
                {
                    QString message="The file ["+suggestedName+"] can't be saved.<br>Invalid direction or file already exists";
                    QMessageBox::critical(this,"File not saved",message,QMessageBox::Ok);
                }
            }
        }
    }
    else //not ok
    {
        QString message="An error occurs during the ["+pathOfFile+"] file generation";
        QMessageBox::critical(this,"File not correctly generated",message);
    }

    delete tsvFile;
    tsvFile=NULL;
    return true;
}

bool DataAcquisition::writeSample(uint32 _sampleNb)
{   //Write a sample in the TSV file
    bool ok=true;
    // Central Sample value :
    vector<uchar> centralSample=peek12ucharFromVector(tableOfDownloadedValues[0],_sampleNb);
    if(centralSample.size()!=12) return false;  //error

    //Add date :
    QString lineToWrite = QString::number(centralSample[2],16).rightJustified(2,'0')+"/"   //day
            +QString::number(centralSample[1],16).rightJustified(2,'0')+"/"      //month
            +QString::number(centralSample[0],16).rightJustified(2,'0')+" "        //year
            +QString::number(centralSample[3],16).rightJustified(2,'0')+":"        //hour
            +QString::number(centralSample[4],16).rightJustified(2,'0')+":"        //min
            +QString::number(centralSample[5],16).rightJustified(2,'0');        //sec

    //Insert voltage of sample :
    uint32 nbOfMeasures=(centralSample[6]&0x3)+((centralSample[8]&0x3) * 4)+((centralSample[10]&0x3) * 16);
    if(nbOfMeasures==0) //restart after a power cut
        lineToWrite+=("\t0.0\t0.0\t0.0");
    else
    {
        uint32 voltageRms;
        double voltage;
        for(int currentPhase=0; currentPhase<3; currentPhase++) //phase 0, 1, 2
        {
            if(nbOfMeasures>0)
            {
                voltageRms=centralSample[2*currentPhase+7]*256 + (centralSample[2*currentPhase+6]&0xFC);
                voltage=sqrt(44.74 / nbOfMeasures * voltageRms);
                if(voltage<10) voltage = 0;
                lineToWrite+=("\t"+QString::number(voltage,'f',1));
            }
        }
    }
/*

    //Add date :
    QString lineToWrite = QString::number(tableOfDownloadedValues[0][2+_sampleNb],16).rightJustified(2,'0')+"/"   //day
            +QString::number(tableOfDownloadedValues[0][1+_sampleNb],16).rightJustified(2,'0')+"/"      //month
            +QString::number(tableOfDownloadedValues[0][0+_sampleNb],16).rightJustified(2,'0')+" "        //year
            +QString::number(tableOfDownloadedValues[0][3+_sampleNb],16).rightJustified(2,'0')+":"        //hour
            +QString::number(tableOfDownloadedValues[0][4+_sampleNb],16).rightJustified(2,'0')+":"        //min
            +QString::number(tableOfDownloadedValues[0][5+_sampleNb],16).rightJustified(2,'0');        //sec

    //Insert voltage of sample :
    uint32 nbOfMeasures=(tableOfDownloadedValues[0][6+_sampleNb]&0x3)+((tableOfDownloadedValues[0][8+_sampleNb]&0x3) * 4)+((tableOfDownloadedValues[0][10+_sampleNb]&0x3) * 16);
    if(nbOfMeasures==0) //restart after a power cut
        lineToWrite+=("\t0.0\t0.0\t0.0");
    else
    {
        uint32 voltageRms;
        double voltage;
        for(int currentPhase=0; currentPhase<3; currentPhase++) //phase 0, 1, 2
        {
            if(nbOfMeasures>0)
            {
                voltageRms=tableOfDownloadedValues[0][2*currentPhase+7+_sampleNb]*256 + (tableOfDownloadedValues[0][2*currentPhase+6+_sampleNb]&0xFC);
                voltage=sqrt(44.74 / nbOfMeasures * voltageRms);
                if(voltage<10) voltage = 0;
                lineToWrite+=("\t"+QString::number(voltage,'f',1));
            }
        }
    }*/
    //Set the offsetPerLines Array
   //uint32 offsetPerLine[moduleList.size()];
   /* for(uint32 moduleIndex=0; moduleIndex<moduleList.size(); moduleIndex++)
    {
        if(moduleList.at(moduleIndex).nbOfLines>0)//si nbOfLines=0 : module wireless sans sensor, donc pas de donnes  rcuprer
        {
            offsetPerLine[moduleIndex] = tableOfDownloadedValues[moduleIndex+1].size()/moduleList.at(moduleIndex).nbOfLines;
        }
    }*/
    // To cover each module :
    for(uint32 moduleIndex=0; moduleIndex<moduleList.size(); moduleIndex++)
    {
        //fetching module informations :
        Module currentModule=moduleList.at(moduleIndex);

        //To cover each line of sensor :
        for(int lineIndex=0; lineIndex<currentModule.nbOfLines; lineIndex++)
        {
            //fetching values to compute (group of 12 uchar) :
            uint32 samplePosition=_sampleNb+lineIndex*offsetPerLine[moduleIndex];
            //Be careful of line placing in the vector : should be next module of 64

            //To cover each channel :
            for(int currentChannel=0; currentChannel<6; currentChannel++)
            {   //0 to 5 (measure module) or 0 to maxSensor for wireless module
                float value;
                int16 data=peekUint16FromVectorBigEndian(tableOfDownloadedValues[moduleIndex+1],currentChannel*2+samplePosition);
                if(currentModule.typeOfModule==wirelessModule)
                {
                    uint16_t current_index=lineIndex*6+currentChannel;
                    if((uint16)data != 0x8000 && (uint16)data!= 0x7FFE && (uint16)data != 0x7FFF
                            && currentModule.sensorList[current_index].serialNb!=0)
                    {
                        if(currentModule.sensorList[current_index].channelType<0x80)
                            value=float(data+0.5)*float(currentModule.sensorList[current_index].coefficient);

                        else    //0x80,81,90,91,92
                        {
                        value=data;//peekUint16FromVectorBigEndian(tableOfDownloadedValues[moduleIndex+1],(currentChannel<<1)+samplePosition);
                        //uint16 data_old;
                       // if(_sampleNb>0)data_old=peekUint16FromVectorBigEndian(tableOfDownloadedValues[moduleIndex+1],currentChannel*2+((_sampleNb-1)*12+lineIndex*offsetPerLine[moduleIndex]));
                       // else data_old=new_data;
                       value=uint32(((uint16)value)+0.5)*float(currentModule.sensorList[current_index].coefficient);

                        }
                        lineToWrite+=("\t"+QString::number(value,'f',currentModule.sensorList[current_index].displayPrecision));
                    }
                    else
                    {
                        lineToWrite+="\t";   // Error in the value (no update/invalid)
                    }
                }

                else if(currentModule.typeOfModule==powerModule2)
                {
                    float power=data*currentModule.mulCoeff[currentChannel]/currentModule.divCoeff[currentChannel];
                    data=peekUint16FromVectorBigEndian(tableOfDownloadedValues[moduleIndex+1],currentChannel*2+samplePosition+(tableOfDownloadedValues[moduleIndex+1].size()>>1));
                    float reactive_power=data*currentModule.mulCoeff[currentChannel]/currentModule.divCoeff[currentChannel];

                    //if(value>-2.5 && value<2.5) value = 0;
                    if(currentModule.divCoeff[currentChannel]>1)
                    {
                        if(currentModule.divCoeff[currentChannel]>100)  lineToWrite+=("\t"+QString::number(power,'f',2)+"\t"+QString::number(reactive_power,'f',2));
                        else  lineToWrite+=("\t"+QString::number(power,'f',1)+"\t"+QString::number(reactive_power,'f',1));
                    }
                    else lineToWrite+=("\t"+QString::number(power)+"\t"+QString::number(reactive_power));
                }

                else
                {
                    value=data;
                    value=value*currentModule.mulCoeff[currentChannel]/currentModule.divCoeff[currentChannel];
                    if(uint32(currentModule.serialNb>>8)!=0x18)    // modules with sn= ..18xx are not for energy : display all
                    {
                        if(value>-2.5 && value<2.5) value = 0;
                    }
                    if(currentModule.divCoeff[currentChannel]>1)
                    {
                        if(currentModule.divCoeff[currentChannel]>100)  lineToWrite+=("\t"+QString::number(value,'f',2));
                        else  lineToWrite+=("\t"+QString::number(value,'f',1));
                    }
                    else lineToWrite+=("\t"+QString::number(value));
                }
            }// end of each channel
        }// end of each line
    }//end of each module
    //Write line
    tsvFileStream<<"\r\n"<<lineToWrite;
    return ok;
}

//************
//  Debug
//************

void DataAcquisition::debugDisplayTable()
{
    //Used for debug only : display the table of downloaded Values
    qDebug()<<"tableOfDownloadValues :";
    QString line;
    for(uint32 x=0; x<tableOfDownloadedValues.size(); x++)
    {
        qDebug()<<"Module nb "<<x;
        line.clear();
        for(uint32 y=0; y<tableOfDownloadedValues[x].size(); y++)
        {
            line+=QString::number(tableOfDownloadedValues[x][y],16).rightJustified(2,'0')+" ";
            if((y+1)%12==0)
            {
                qDebug()<<"("<<(y-11)<<")"<<line;
                line.clear();
            }
        }
        qDebug()<<line;
    }
}

