#include "frame_relay.h"

frame_relay::frame_relay(ManageSerialPort *_serialPort, QWidget *parent) :
    QWidget(parent)
{
    serialCom = _serialPort;
    setWindowIcon(QIcon("icon/radio.png"));
    setWindowTitle("Wireless Frame relay Configuration");
    setWindowModality(Qt::ApplicationModal);
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/RELAY_WIN_X")&&
            settingFile->contains("WINDOW_POS/RELAY_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/RELAY_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/RELAY_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    fillWindow();

}

void frame_relay::fillWindow(){
    bool existing=false;
    nb_of_sensor=1;
    nb_of_plug=0;
    uint8_t nb_of_line=0;
    bool ok=serialCom->readBytesAtWithRetry(serialCom->address_API,0x0100008D,1,3);
    if(!ok){
        nb_of_line=255;
    }
    else nb_of_line=serialCom->getReceivedData()[0];
    if(nb_of_line<8)
    {
        list_sensor[0]=serialCom->address_API;
        for (int i=0;i<nb_of_line;i++){
            if(serialCom->readBytesAtWithRetry(serialCom->address_API,0x010000A0+12*i,2,3))
            {
                for (int j=0;j<6;j++)
                {
                    if(peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j)!=0){
                        existing=false;
                        for(int k=1;k<nb_of_sensor;k++){
                            if(peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j)==list_sensor[k])existing=true;
                        }
                        if(!existing) list_sensor[nb_of_sensor++]=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j);
                    }

                    if(((peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j))&0x8000)!=0){
                        existing=false;
                        for(int k=0;k<nb_of_plug;k++){
                            if(peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j)==list_plug[k])existing=true;
                        }
                        if(!existing) list_plug[nb_of_plug++]=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),2*j);
                    }
                }
            }
        }

    }
    else {
        QMessageBox::critical(this,"Unsupported radio module","The radio module doesn't support frame relay");
        close();
        return;
    }
    setMinimumWidth(500);
    setMinimumHeight(500);
    verticalLayout = new QVBoxLayout;
    //QHBoxLayout *h_layout1=new QHBoxLayout;
    table_sensor=new QTableWidget(nb_of_sensor,nb_of_plug);

    table_sensor->setStyleSheet("selection-background-color: rgba(0, 0, 0, 0);"
    "selection-color: rgba(0, 0, 0, 255);");
    table_sensor->setEditTriggers(QAbstractItemView::NoEditTriggers);
    table_sensor->connect(table_sensor,SIGNAL(cellClicked(int,int)),this,SLOT(cell_clicked(int,int)));
    table_sensor->connect(table_sensor->horizontalHeader(),SIGNAL(sectionClicked(int)),this,SLOT(column_clicked(int)));
    for(int i=0;i<nb_of_plug;i++)
    {
        QTableWidgetItem *sensor_sn = new QTableWidgetItem(QString::number(list_plug[i],16).rightJustified(4,'0'));
        table_sensor->setHorizontalHeaderItem(i,sensor_sn);
        table_sensor->setColumnWidth(i,50);
    }

    for(int i=0;i<nb_of_sensor;i++)
    {
        QTableWidgetItem *sensor_sn = new QTableWidgetItem("  "+QString::number(list_sensor[i],16).rightJustified(4,'0'));
        table_sensor->setVerticalHeaderItem(i,sensor_sn);
    }
    //*********Empty Tab
    for(int i=0;i<nb_of_plug;i++)
    {
        for(int j=0;j<nb_of_plug;j++)
        {
            QTableWidgetItem *sensor_r= new QTableWidgetItem("");
            table_sensor->setItem(j,i,sensor_r);
        }

    }

    fill_table();
    verticalLayout->addWidget(table_sensor);
    //verticalLayout->addLayout(h_layout1);
    QHBoxLayout *h_layout2=new QHBoxLayout;
    refresh_button= new QPushButton("Refresh");
    connect(refresh_button,SIGNAL(clicked()),this,SLOT(refresh_clicked()));
    reset_button= new QPushButton("Reset");
    connect(reset_button,SIGNAL(clicked()),this,SLOT(reset_clicked()));
    h_layout2->addWidget(reset_button);
    h_layout2->addWidget(refresh_button);

    verticalLayout->addLayout(h_layout2);
    setLayout(verticalLayout);
    setWindowModality(Qt::ApplicationModal);

}

void frame_relay::fill_table(){

    for(int i=0;i<nb_of_plug;i++)
    {
        for(int j=0;j<nb_of_sensor;j++)
        {
            QTableWidgetItem *sensor_r= new QTableWidgetItem("");
            table_sensor->setItem(j,i,sensor_r);
            tab_relay[j][i]=false;
        }

    }
    for(int i=0;i<nb_of_plug;i++)
    {
        QTableWidgetItem *sensor_sn = new QTableWidgetItem("  "+QString::number(list_plug[i],16).rightJustified(4,'0'));
        table_sensor->setHorizontalHeaderItem(i,sensor_sn);
    }

    for(int i=0;i<nb_of_sensor;i++)
    {
        QTableWidgetItem *sensor_sn = new QTableWidgetItem("  "+QString::number(list_sensor[i],16).rightJustified(4,'0'));
        table_sensor->setVerticalHeaderItem(i,sensor_sn);
    }

    for(int i=0;i<nb_of_plug;i++)
    {
        if(serialCom->read_data_relay(0x29DE,2,list_plug[i]))
        {
            if(serialCom->getAllReceivedData()[1]==0xFF){
                    for(int k=1;k<nb_of_sensor;k++){
                        QTableWidgetItem *sensor_r= new QTableWidgetItem(".");
                        tab_relay[k][i]=true;
                        table_sensor->setItem(k,i,sensor_r);
                    }
            }
            else{
                for(int j=0; j<8;j++){
                    for(int k=0;k<nb_of_sensor;k++){
                        if(peekUint16FromUcharTabLittleEndian(serialCom->getAllReceivedData(),2*j+1)==list_sensor[k]) {
                            QTableWidgetItem *sensor_r= new QTableWidgetItem(".");
                            tab_relay[k][i]=true;
                            //tab_sensor[k]=true;
                            table_sensor->setItem(k,i,sensor_r);
                        }
                    }
                }
            }

            if(serialCom->read_data_relay(0x28,1,list_plug[i]))
            {
                for(int j=0; j<19;j++){
                    for(int k=0;k<nb_of_sensor;k++){
                        if(peekUint16FromUcharTabLittleEndian(serialCom->getAllReceivedData(),3*j+1)==list_sensor[k]) {
                            QTableWidgetItem *sensor_r= new QTableWidgetItem("");
                            if(tab_relay[k][i])
                                sensor_r->setText(".     "+QString::number(serialCom->getAllReceivedData()[3*j+3]*99/255,10).rightJustified(2,'0',true));
                            else
                                sensor_r->setText("       "+QString::number(serialCom->getAllReceivedData()[3*j+3]*99/255,10).rightJustified(2,'0',true));
                            table_sensor->setItem(k,i,sensor_r);
                        }
                    }
                }
            }
        }
        else {
            QTableWidgetItem *module_r= new QTableWidgetItem("");
            module_r->setText("     ?");
            table_sensor->setItem(0,i,module_r);
        }
    }


    for(int i=0;i<nb_of_sensor;i++)
    {
        int sensor_cnt=0;
        for(int j=0;j<nb_of_plug;j++)
        {
            if(tab_relay[i][j]) sensor_cnt++;
        }
        if(sensor_cnt==1){
            QTableWidgetItem *sensor_sn = new QTableWidgetItem(". "+QString::number(list_sensor[i],16).rightJustified(4,'0'));
            table_sensor->setVerticalHeaderItem(i,sensor_sn);
        }
        else if(sensor_cnt>1){
            QTableWidgetItem *sensor_sn = new QTableWidgetItem("! "+QString::number(list_sensor[i],16).rightJustified(4,'0'));
            table_sensor->setVerticalHeaderItem(i,sensor_sn);
        }
    }
}


void frame_relay::cell_clicked(int row, int col){
    if(row>=1){
        uint16_t data_to_send[8];
        if(serialCom->read_data_relay(0x29DE,2,list_plug[col])){
            if(serialCom->getAllReceivedData()[1]==0xFF){
               for(int i=0;i<8;i++)  data_to_send[i]=0;
            }
            else{
                for(int i=0;i<8;i++)
                {
                    data_to_send[i]=peekUint16FromUcharTabLittleEndian(serialCom->getAllReceivedData(),i*2+1);
                }

                if(tab_relay[row][col]){
                    bool sensor_encountered=false;
                    for(int i=0;i<8;i++)
                    {
                        if(data_to_send[i]==list_sensor[row]) sensor_encountered=true;
                        if(sensor_encountered){
                            if(i<7)data_to_send[i]=data_to_send[i+1];
                            else data_to_send[i]=0;
                        }
                    }
                }
                else
                {
                    for(int j=7;j>0;j--) data_to_send[j]=data_to_send[j-1];
                    data_to_send[0]=list_sensor[row];
                }
            }
        serialCom->write_data_relay(0x26,data_to_send,list_plug[col]);
        waitFor(50);
        }
    }
    fill_table();
}

void frame_relay::column_clicked(int col){

    uint16_t data_to_send[8];
    if(serialCom->read_data_relay(0x29DE,2,list_plug[col])){
        for(int i=1; i<8;i++) data_to_send[i]=0;
        if(serialCom->getAllReceivedData()[1]==0xFF){
            data_to_send[0]=0;


        }
        else{
            data_to_send[0]=0xFF00;
        }
        serialCom->write_data_relay(0x26,data_to_send,list_plug[col]);
        waitFor(50);
    }
    fill_table();
}

void frame_relay::refresh_clicked(){
    fill_table();
}

void frame_relay::reset_clicked(){
    for(int j=0;j<nb_of_plug;j++)
    {
    uint16_t data_to_send[8];
    for(int i=0; i<8;i++) data_to_send[i]=0;

    serialCom->write_data_relay(0x26,data_to_send,list_plug[j]);
        waitFor(50);
    }
        fill_table();

}

void frame_relay::closeEvent(QCloseEvent *event)
 {
    setVisible(false);
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    settingFile->setValue("WINDOW_POS/RELAY_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/RELAY_WIN_Y",QVariant(QString::number(this->pos().y())));
    emit relayClosed();
}
