#ifndef GPRSCONFIG_H
#define GPRSCONFIG_H

#include "manageserialport.h"
#include "refreshtimer.h"
//Generic address for GPRS = 0x00FF0000
#define GPRS_SN 0x00FF0000

class GprsConfig : public QWidget
{
    Q_OBJECT
public:
    explicit GprsConfig(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, vector<Module> *_moduleList, QWidget *parent = 0);


signals:
    void closingWindow();
    void timeout_cfg();

public slots:
    void mousePressEvent(QMouseEvent *)
    {
        emit timeout_cfg();
    }
    void initWindow();
    void closeEvent(QCloseEvent *event);
    void setGprsModuleListFromCentral();
    void saveConfiguration();
    void sendTimeToCentral();
    void resendAllData();
    void timerSlot();
    void saveCommand();
    void sendCommand();
    void newCommandSelected(int _newCommand);
    void updateLogView();
    void save_IMEI();
    void clr_IMEI();
    void force_send();
        bool fetchGprsInfo();


private :
    bool no_com;
    QSettings *settingFile;
    ManageSerialPort *serialCom;
    RefreshTimer *commonRefreshTimer;
    vector<Module> *centralModuleList;
    bool firstTimeOpened;
    uchar logPointer;
    bool saveIMEI;


    bool updateGprsTime();
    void updateModuleListFromCentral();
    void updateGprsModuleListFromGprs();

    void fillWindow();
    QHBoxLayout *windowLayout;
    QVBoxLayout *normalConfigLayout;
        QPlainTextEdit *moduleListView;
        QPlainTextEdit *gprsModuleListView;
        QPushButton *shiftModules;
        QSpinBox *callTimeHours;
        QSpinBox *callTimeMinutes;
        QCheckBox *checkHeader;
        QLabel *gprsTime;
        QPushButton *centralTimeButton;
        QLineEdit *ftpDirectory;
        QLineEdit *ftps;
        QLineEdit *ftpu;
        QLineEdit *ftpp;
        QLineEdit *apn;
        QLineEdit *apnu;
        QLineEdit *apnp;

        QCheckBox *checkAPN;

        QPushButton *saveConfigButton;
        QLabel *dataPointer;
        QPushButton *sendRemainingData;
        QLabel *versionLabel;
    QVBoxLayout *advancedConfigLayout;
        QPlainTextEdit *logView;
        QComboBox *preselectedCommands;
        QLineEdit *commandLineEdit;
        QPushButton *sendCommandLine;
        QPushButton *saveCommandLine;
};

#endif // GPRSCONFIG_H
