#ifndef MANAGEMODULE_H
#define MANAGEMODULE_H

#include "manageserialport.h"
#include "dataacquisition.h"
#include "wirelessmodule.h"
#include "powermodule.h"
#include "presstempmodule.h"
#include "refreshtimer.h"
#define NB_RETRY_TO_FIND_MODULES 2  //nb of tries with no module found before ending search

class ManageModule : public QWidget
{
    Q_OBJECT
public:
    ManageModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, vector<Module> *_moduleList, QWidget *parent =0);
    ~ManageModule();
    bool endOfFindingModules;

signals:
    void eraseCentralInTitle();
    void restart_timer_right_panel();
    void setadvusermode();
    void ad_ok();


public slots:
    void switchToAdvancedUserMode();
    void enableWindowAndStartTimerAgain();
    void connect_timer();
    void disableWindow();
    void initCentralDetection();
    void closeRightPanel();

private slots:
    void clickOnACell(int row, int column);
    void dataAcquisitionAsked();
    void resetSystem();

    void timerSlot();

private:
    // Attributes
    ManageSerialPort *serialCom;
    RefreshTimer *commonRefreshTimer;
    bool isStarted;
    bool advancedUserMode;
    vector<Module> *moduleList;
    int autodetectRetry;
    int nbRemainingRetryToFindModule;

    bool centralDetected;
    vector< vector< QTableWidgetItem * > > itemTableForWidget;
    uint8_t left_pannel_type;
    enum{
        empty=0,
        dataacq,
        powerm,
        wireless,
        wireless2,
        presst
                  };
    uint8_t pannel_row;
    //Methods
    void fillWindow();
    int fetchModuleDetectionResult();
    void setTableWidget();
    void refreshTableValues();
    bool initModuleDetection();
    bool search_more();

    // print/update the table widget :
    void changeTextOnCell(int position, int channel, QString cellText);
    void fetchModuleValues(int lineNb);
    bool addFoundModuleOnList(uint32 _moduleSN);
    void moduleAnswer(int _moduleNb, bool _answer);
    void blurAllModules(bool blur);

    //tests
    void updateTableLines();
    void updateVoltageValues();

    //GUI elements :
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalModuleLayout;

    QGroupBox *groupBoxModules;
        QTableWidget *tableWidget;
            QTableWidgetItem *moduleSNTableItem;
            QTableWidgetItem *channel1TableItem;
            QTableWidgetItem *channel2TableItem;
            QTableWidgetItem *channel3TableItem;
            QTableWidgetItem *channel4TableItem;
            QTableWidgetItem *channel5TableItem;
            QTableWidgetItem *channel6TableItem;
        QGridLayout *voltageLayout;
            QLabel *voltagePh1Values;
            QLabel *voltagePh2Values;
            QLabel *voltagePh3Values;
        QHBoxLayout *buttonLayout;
            QPushButton *autoDetectButton;
            QPushButton *resetSystemButton;
            //QPushButton *startConfigButton;
            QPushButton *dataButton;
    QStackedLayout *rightPanelLayout;
        QWidget *rightPanelWidget;
        QWidget *emptyRightPanelWidget;
    //for right panel :
    QWidget *emptyWidget;
    QWidget *dataWidget;
};

#endif // MANAGEMODULE_H
