#ifndef MANAGESERIALPORT_H
#define MANAGESERIALPORT_H

#define ALL_MODULE 0x00000000
#define ALL_CENTRAL 0x01000000
#define MAX_SUCCESSIVE_ERROR_COM 10

#define E_READ_FAILED               13
#define E_WRITE_FAILED              14

#include "serialframe.h"
#include <qextserialport.h>
#include <qextserialenumerator.h>
#include "common.h"

#ifdef _TTY_POSIX_
#include <wiringPi.h>
#include <wiringPiI2C.h>
#endif

class ManageSerialPort : public QObject
{
    Q_OBJECT

public:
//Constructors + destructor
    ManageSerialPort();
    ~ManageSerialPort();
    QextSerialPort port;

//General

    //Serial Port
    QDateTime sensor_receive_date_time;
   // QDateTime current_time;
    uint64_t address_API;
    connectionType typeOfConnection;
    bool com_mdl;
    uint16_t rx_source_address;

    bool interrupt_com;

    uint8_t currentpl;


    QByteArray receivedChars;


    int rssi;
    uint32_t rssi_868;
    uint32_t n_rssi_868;
    bool read_serial_data;
    bool no_com;

    void close();
    int getTypeofconnection();
    bool isOpen();
    bool open(QString portname, int _typeOfConnection);
    bool test_xstick();
    bool config_xstick();
    void setBaud111();

   /* bool getErrorCom();
    void deleteErrorCom();*/
    // getters & setters
    uchar *getReceivedData();
    uchar *getAllReceivedData();
    int getReceivedDataLength();
    int getReceivedLength();
    uint32 getCentralSN();
    QString getCentralName();
    bool setModuleName(uint32 _serialNb, uint32 _address, QString _newName);
    bool write_register(QString reg,uint64_t dest_adress,QByteArray value);
    QByteArray read_register(QString reg,uint64_t dest_adress);
    bool update_add_api_868();
    bool send_sensor_firmware(QByteArray datatowrite);
    // read and write
    bool readBytesAt(uint32 _serialNb, uint32 _address, uchar _nbPacketsToRead);
    bool readBytesAtWithRetry(uint32 _serialNb, uint32 _address, uchar _nbPacketsToRead, uint32 nbOfRetry);
    bool writeByteAt(uint32 _serialNb, uint32 _address, uchar _dataToWrite);
    bool writeByteAtWithRetry(uint32 _serialNb, uint32 _address, uchar _dataToWrite, uint32 nbOfRetry);
    bool writeByteAtS(uint32 _serialNb, uint32 _address, uchar _dataToWrite);
    bool writeByteAtWithRetryS(uint32 _serialNb, uint32 _address, uchar _dataToWrite, uint32 nbOfRetry);
    bool write4BytesAt(uint32 _serialNb, uint32 _address, uchar _dataToWrite[4]);
    bool write4BytesAtWithRetry(uint32 _serialNb, uint32 _address, uchar _dataToWrite[4], uint32 nbOfRetry);
    bool writeByteDontCheckResponse(uint32 _serialNb, uint32 _address, uchar _dataToWrite);
    bool sendframe_wdata(uint32_t serialNumber, uchar function, uchar param[16],uint16_t sn_relay);
   // bool sendframe_calibrate(uint32_t serialNumber, uchar function, uchar param[6],uint16_t add_api);
    bool sendframe_wdataack(uint32_t serialNumber, uchar function, uchar param[17],uint16_t add_api);
    //bool read_data_relay(uint16_t function, uint8_t f_size, uint16_t add_API);
   // bool write_data_relay(uint8_t function, uint16_t data_to_send[8],uint16_t add_API);
    QByteArray receive_data_API(uint64_t add_api, uint16_t timeout);
    QByteArray receive_data_API(uint64_t add_api);
    bool receiveFrameAPI_withADD(uint64_t add_api, uint16_t timeout);
    bool receiveFrameAPI_withADD(uint64_t add_api);

    bool read_data_relay(uint16_t function, uint8_t f_size,uint16_t add_API);
    bool write_data_relay(uint8_t function, uint16_t data_to_send[8],uint16_t add_API);


    //Wireless Module
    int readframe_wmodule(uint16_t add_api,uint16_t timeout);
//    int receiveFramedlModuleWireless(uint16_t add_api,uint16_t timeout);

        uint8_t read_chan();

    // Ethernet Module
    QByteArray ReadEthernetModuleRam(uint16 Address,uint32 nb);
    QByteArray ReadEthernetModuleFlash(uint32 Address,uint32 nb);
    bool WriteEthernetModuleRam(uint16 Address,QByteArray DataToWrite);
    bool WriteEthernetModuleSpecial(uchar Command, QByteArray DataToWrite);
    bool write_serial_data(QByteArray DataToWrite,uint8_t _n_try);
    bool centralDetection();

    // Debug
    void displayReception();
    void displayRegPerf();
    void setBaudrate();
    void init_no_com();

    int program_dongle(QString filename);
    uint8_t get_dongle_version();

signals:
    void errorCom();
    void noErrorCom();
    void comclose();
    void comopen();
    void update_program_status(QString status);

//public slots:
// void refresh_serial();

private:
        QTimer *timer1;
     bool button_clicked;
    bool flag_timeout_timer1;
    bool flag_stop_rx;
    BaudRateType baudrate;
    int errorComCounter;
    int timeoutCoeffOver4;  // coeff if length >4
    long timeoutBase;        // base value of the timeout
    int receptionTimeout;    //by default 40, set with setReceptionTimeoutFromLength
    //QSettings *settingFile;
    vector< vector< int> > perfRegister;

    uint32 centralSN;
    char centralName[17];
    SerialFrame transmissionFrame;
    SerialFrame receptionFrame;
    bool baud_xstick;

    bool sendFrame();
    bool sendQByteArray(QByteArray _arrayToSend);
    bool sendReadFrame(uint32 _serialNumber, uint32 _address, uchar _nbToRead);
    bool sendWriteFrame(uint32 _serialNumber, uint32 _address, uchar _dataToWrite);
    bool sendWriteFrameS(uint32 _serialNumber, uint32 _address, uchar _dataToWrite);
    bool sendWriteFrame4(uint32 _serialNumber, uint32 _address, uchar _dataToWrite[4]);

    bool receiveFrame();
    bool receiveFrame_std();


    void setTimeoutConfig();
   /* void errorComOccurs();
    void resetErrorCom();*/

private slots:
    void stop_rx();

#ifndef NO_GUI
    void onDeviceRemoved(const QextPortInfo &a);
    void onDeviceAttached(const QextPortInfo &a);
#endif

    void timeout_timer1();


};

#endif // MANAGESERIALPORT_H
