#ifndef POWERMODULE_H
#define POWERMODULE_H

#include "manageserialport.h"
#include "waveform.h"
#include "refreshtimer.h"
class PowerModule : public QWidget
{
    Q_OBJECT
public:
    explicit PowerModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, Module *_powModule, int _channel, QWidget *parent = 0, bool _advancedUserMode = false);
    ~PowerModule();

signals:
    void backToMainWindow();

public slots:
    void startTimerAgain();
    void setadvance();

private slots:
    void backToManageModule();

    //refresh
    void timerSlot();

    //send new values
    void channelChanged(int _newChannel);
    void channelNameChanged();
    void mulCoeffChanged(int _newCoeff);
    void divCoeffChanged(int _newCoeff);
    void clearKwhClicked();
    void invertSign(bool _invert);
    void phaseChanged();

    void waveformClicked();
    void returnfromwaveform();

private:
    bool updating_phase;
    ManageSerialPort *serialCom;
    RefreshTimer *commonRefreshTimer;
    Module *powModule;
    int currentChannel;     // Be careful : start with 1
    bool advancedUserMode;
    bool updateModuleName();
    bool updatePhaseAndInversion();
    bool sendPhaseAndInversion();
    int phase;
    bool updateChannelInfo();
    bool channelInfoUpdated;
    waveform *waveformwindow;
    //GUI elements :
    void fillWindow();
    QVBoxLayout *globalLayout;
    QGroupBox *groupBoxPowerModule;
    QVBoxLayout *verticalLayout;
        QLabel *moduleSN;
        //QComboBox *channelComboBox;
        QButtonGroup *chan_b_group;
        QRadioButton *chan1;
        QRadioButton *chan2;
        QRadioButton *chan3;
        QRadioButton *chan4;
        QRadioButton *chan5;
        QRadioButton *chan6;
        QLineEdit *channelNameEdit;
        QFrame *valueFrame;
                QLabel *powerValue;
                QLabel *apparentPowerValue;
                QLabel *reactivePowerValue;
                QLabel *voltageValue;
                QLabel *currentValue;
                QLabel *energyValue;
                QLabel *reactiveEnergyValue;
                QLabel *powerFactorValue;
        QSpinBox *mulCoeff;
        QSpinBox *divCoeff;
        QCheckBox *invertBox;
        QGroupBox *phaseGroupBox;
            QRadioButton *phase1;
            QRadioButton *phase2;
            QRadioButton *phase3;
            QRadioButton *not_configured;
            QCheckBox *phase_auto;
        QHBoxLayout *buttonLayout;
            QPushButton *returnButton;
            QPushButton *clearKwhButton;
            QPushButton *waveformButton;
};

#endif // POWERMODULE_H
