#include "serialframe.h"

//***************************
// Constructors / destructor
//***************************

SerialFrame::SerialFrame()
{
    //Default constructor : empty frame
    frame=NULL;
    frame= new uchar[1300];
    length=0;
}

SerialFrame::~SerialFrame()
{
    //Destructor
    delete[] frame;
    frame=NULL;
}


//***************************
// Copy methods
//***************************


uchar *SerialFrame::copy(const uchar *_frame, int _length)
{
    //Copy method : copy char one by one
    uchar *copyFrame = new uchar[_length + 1]; // +1 to stock \0
    for (int i = 0 ; i < _length ; i++)
    {
        copyFrame[i] = _frame[i];
    }
    copyFrame[_length] = '\0';
    return copyFrame;
}

//***************************
// Allocate / erase
//***************************

void SerialFrame::allocate(int _length)
{
    //Allocate memory for frame, with length specified
    frame=NULL;
    frame=new uchar[_length + 1];
}

void SerialFrame::erase()
{
    //Erase frame by putting all to 0xFF, prevent from using a previous frame
    if(frame!=NULL)
    {
        for(int i=0;i<1300; i++) frame[i]=0xFF;
        length=0;
    }
}

//***************************
// Getters
//***************************

uchar * SerialFrame::getFrame() const{   //return the uchar array of frame
    return frame;
}

int SerialFrame::getLength() const{     //return the length used
    return length;
}

uchar *SerialFrame::getData(){      //return the data read (from 10th char)
    return (frame+10);  //data start at 10
}

uchar *SerialFrame::getAllData(){      //return the data read (from 10th char)
    return (frame);
}

int SerialFrame::getDataLength(){   //return data read length
    return (length-13); //13 = everything but data
}

//***************************
// Setters
//***************************

void SerialFrame::changeFrame(char *_frame, int _length)
{
    //replace chars by _frame
    if(frame!=NULL)
    {
        for(int i=0; i<_length; i++) frame[i]=uchar(_frame[i]);
    }
    else
    {
        frame=copy(reinterpret_cast<const uchar *>(_frame),_length);
    }
    length=_length;
}


//***************************
// Process CRC
//***************************

uint16 SerialFrame::computeCrc(uchar *_frame,int start_pos, int _length)
{
    // Compute the CRC value from the frame
    static uint16 fcsTab[256] = {0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf, 0x8c48, 0x9dc1, 0xaf5a,
                                 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7, 0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c,
                                 0x75b7, 0x643e, 0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876, 0x2102,
                                 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd, 0xad4a, 0xbcc3, 0x8e58, 0x9fd1,
                                 0xeb6e, 0xfae7, 0xc87c, 0xd9f5, 0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5,
                                 0x453c, 0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974, 0x4204, 0x538d,
                                 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb, 0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868,
                                 0x99e1, 0xab7a, 0xbaf3, 0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
                                 0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72, 0x6306, 0x728f, 0x4014,
                                 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9, 0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3,
                                 0x8a78, 0x9bf1, 0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738, 0xffcf,
                                 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70, 0x8408, 0x9581, 0xa71a, 0xb693,
                                 0xc22c, 0xd3a5, 0xe13e, 0xf0b7, 0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76,
                                 0x7cff, 0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036, 0x18c1, 0x0948,
                                 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e, 0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e,
                                 0xf2a7, 0xc03c, 0xd1b5, 0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
                                 0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134, 0x39c3, 0x284a, 0x1ad1,
                                 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c, 0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1,
                                 0xa33a, 0xb2b3, 0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb, 0xd68d,
                                 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232, 0x5ac5, 0x4b4c, 0x79d7, 0x685e,
                                 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a ,0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238,
                                 0x93b1, 0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9, 0xf78f, 0xe606,
                                 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330, 0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3,
                                 0x2c6a, 0x1ef1, 0x0f78};
    uint16 fcs=0xffff;
    while (_length--)		// while length >0
    fcs = (fcs >> 8) ^ fcsTab[(fcs ^ _frame[start_pos++]) & 0xff];
    return (fcs);
}

bool SerialFrame::checkCRC()
{
    if(length>6)
    {
        // Crc value
        uint16 crcReceived ;
        lo(crcReceived)= frame[length-3];
        hi(crcReceived)=frame[length-2];

        // check the CRC value
        uint16 crcValue=computeCrc(frame,1,length-4);
        if(crcReceived!= (uint16_t)~crcValue)
        {
            qDebug()<<"CRC error ! (received "<<QString::number(crcReceived,16)<<", computed "<<QString::number(~crcValue,16)<<")";
            return false;
        }
        return true;
    }
    else
    {
        qDebug()<<"(length<6)";
        return false;
    }
}


void SerialFrame::check_special_char_in_received_frame(int start_pos,int end_pos)
{
    uchar frameWithoutEscapeChar [length];
    int lengthBeforeDelete=length;
    int newIndex=1;
    for(int index=start_pos;index<end_pos-1;index++) //all frame except start&end
    {
        if((frame[index]==0xC0)||(frame[index]==0xC1))
            return;
        if(frame[index]==0x7D)
        {
            index++;
            frameWithoutEscapeChar[newIndex]=(frame[index] xor 0x20);
            newIndex++;
            length--;
        }
        else    //copy only
        {
            frameWithoutEscapeChar[newIndex]=frame[index];
            newIndex++;
        }
    }

    if(length!=end_pos)  //copy if at least one special char
    {
        for(int index=1;index<length-1;index++) frame[index]=frameWithoutEscapeChar[index];
        frame[length-1]=frame[lengthBeforeDelete-1];    //add c1
    }
}

void SerialFrame::check_special_char_in_frame_to_send()
{
    uchar frameWithoutSpecialChar[(2*length)+1];
    int lengthBefore=length;
    int newIndex=1;
    for(int index=1;index<length-1;index++)
    {
        if((frame[index]==0x7D)||(frame[index]==0xC0)||(frame[index]==0xC1))
        {
            frameWithoutSpecialChar[newIndex]=0x7D;
            newIndex++;
            frameWithoutSpecialChar[newIndex]=(frame[index] xor 0x20);
            newIndex++;
            length++;
        }
        else    //copy only
        {
            frameWithoutSpecialChar[newIndex]=frame[index];
            newIndex++;
        }
    }
    if(length!=lengthBefore)  //copy if at least one special char
    {
        for(int index=1;index<length-1;index++) frame[index]=frameWithoutSpecialChar[index];
    }

}

//***************************
//  Frame validation
//***************************


int SerialFrame::validateReceivedwdata()
{
    // This method validate the received frame by testing with SN, address and CRC.
    int outputError=0;
    uint8_t rssi_relay=0xFF;
    bool frame_relayed=false;
    // check start/end char

    if((frame[length-2]==0xC1)&&(frame[length-1]!=0xC1))
    {
        rssi_relay=frame[length-1];
        if(rssi_relay==0xFF) rssi_relay=0xFE;
        //qDebug()<<"frame relayed";
        frame_relayed=true;
        length--;
    }
    if(frame[0]!=0xC0 || frame[length-1]!=0xC1)
    {
        outputError=1;
        qDebug()<<"error of start/end char";
    }

    // Delete escape char (to process next steps), by copy
    check_special_char_in_received_frame(1,length);

    if(!checkCRC())
    {
        outputError=2;
        qDebug()<<"CRC error";
    }
    length++;
    if(frame_relayed) frame[length-1]=rssi_relay;
    else frame[length-1]=0xFF;
    if(length!=54) outputError=3;
    if(outputError)
    {
     display();
    }

    return outputError;
}


int SerialFrame::validateReceived(uint32 sendSerialNb, uint32 sendAddress,uchar req_length)
{
    // This method validate the received frame by testing with SN, address and CRC.
    int outputError=0;
    // check start/end char


    // Delete escape char (to process next steps), by copy
    check_special_char_in_received_frame(1,length);

    if(length!=0x0d+8*req_length)
    {
        outputError=9;
    }

    if(frame[0]!=0xC0 || frame[length-1]!=0xC1)
    {
        outputError=1;
        qDebug()<<"error of start/end char";
    }

    // check CRC value
    if(!checkCRC())
    {
        outputError=2;
        display();
        qDebug()<<"CRC error";
    }

    // check function == 0x06 (response)
    if(frame[5]!=0x06)
    {
        outputError=3;
        qDebug()<<"not response function";
    }

    // check SN (partially)
    if((frame[1]!=4)&&(frame[1]!=(sendSerialNb >> 24))) //TODO : sendSerial!=4 ??
    {
         outputError=4;
      //   qDebug()<<"not the same MS Byte in SN (or =4)";
    }

    //check address
    if(sendAddress!=peekUint32FromUcharTabLittleEndian(frame,6))
    {
        outputError=8;
      //  qDebug()<<"not the same address";
    }
    return outputError;
}

//***************************
// Creation of frame
//***************************

void SerialFrame::makeReadFrame(uint32 serialNb, uint32 address, uchar nbToRead)
{
    // Edit a frame for reading - function 8
    if(nbToRead>8) nbToRead=8;  //TODO : bluetooth ?
    QByteArray _data;
    _data.append(hhh(serialNb));
    _data.append(hh(serialNb));
    _data.append(hi(serialNb));
    _data.append(lo(serialNb));
    _data.append(0x08);
    _data.append(hhh(address));
    _data.append(hh(address));
    _data.append(hi(address));
    _data.append(lo(address));
    _data.append(nbToRead);
    switch(typeOfConnection)
    {
        case radio802:
            makeFrameAPI(_data,address_API );
            break;
        case radio868:
            makeFrameAPI868(_data,address_API);
            break;
        default:
            makeFrame(_data);
    }

}

void SerialFrame::makeWriteFrame(uint32 serialNb, uint32 address, uchar value)
{
    // Edit a frame for writing - function 7
    QByteArray _data;
    _data.append(hhh(serialNb));
    _data.append(hh(serialNb));
    _data.append(hi(serialNb));
    _data.append(lo(serialNb));
    _data.append(0x07);
    _data.append(hhh(address));
    _data.append(hh(address));
    _data.append(hi(address));
    _data.append(lo(address));
    _data.append(value);
    switch(typeOfConnection)
    {
        case radio802:
            makeFrameAPI(_data,address_API );
            break;
        case radio868:
            makeFrameAPI868(_data,address_API);
            break;
        default:
            makeFrame(_data);
    }
}

void SerialFrame::makeWriteFrameS(uint32 serialNb, uint32 address, uchar value)
{
    // Edit a frame for writing - function 7
    QByteArray _data;
    _data.append(hhh(serialNb));
    _data.append(hh(serialNb));
    _data.append(hi(serialNb));
    _data.append(lo(serialNb));
    _data.append(0x08);
    _data.append(hhh(address));
    _data.append(hh(address));
    _data.append(hi(address));
    _data.append(lo(address));
    _data.append(value);
    switch(typeOfConnection)
    {
        case radio802:
            makeFrameAPI(_data,address_API );
            break;
        case radio868:
            makeFrameAPI868(_data,address_API);
            break;
        default:
            makeFrame(_data);
    }
}

void SerialFrame::makeWriteFrame4(uint32 serialNb, uint32 address, uchar value[4])
{
    // Edit a frame for writing - function F4
    QByteArray _data;
    _data.append(hhh(serialNb));
    _data.append(hh(serialNb));
    _data.append(hi(serialNb));
    _data.append(lo(serialNb));
    _data.append(0xF4);
    _data.append(hh(address));
    _data.append(hi(address));
    _data.append(lo(address));
    _data.append(value[0]);
    _data.append(value[1]);
    _data.append(value[2]);
    _data.append(value[3]);
    switch(typeOfConnection)
    {
        case radio802:
            makeFrameAPI(_data,address_API );
            break;
        case radio868:
            makeFrameAPI868(_data,address_API);
            break;
        default:
            makeFrame(_data);
    }
}


void SerialFrame::makeFrame(QByteArray _data)
{
    //create a frame with parameters elements
    erase();    //to be sure
    length=_data.length()+4;  //standard size
    //put start
    frame[0]=START_OF_FRAME; //0xC0

    int index=0; //put data
    for(index=0;index<_data.length();index++)
        frame[index+1]=_data.at(index);
    index++;
    //put CRC
    uint16 computedCrc;
    computedCrc = ~computeCrc(frame,1,_data.length()); // Invert the CRC

    frame[index++] = computedCrc & 0xff; // Put the least sig. value in 1st byte transmitted.
    frame[index++] = (computedCrc>>8) & 0xff;

    //check and replace special chars
    check_special_char_in_frame_to_send();

    //put end
    frame[length-1]=END_OF_FRAME; //0xC1
    frame[length]='\0'; //to close
}
/*
void SerialFrame::makeFrame4(uint32 serialNb, uint32 address, uchar value[4], uchar function)
{
    //create a frame with parameters elements
    erase();    //to be sure
    length=16;  //standard size
    //put start
    frame[0]=START_OF_FRAME;

    //put SN
    placeUint32InUcharTab(serialNb,frame,1);

    //put function
    frame[5]=function;

    //put address
    frame[6]=(address&0xFF0000)>>16;
    frame[7]=(address&0xFF00)>>8;
    frame[8]=address&0xFF;
    //placeUint32InUcharTab(address,frame,6);

    //put value
    frame[9]=value[0];
    frame[10]=value[1];
    frame[11]=value[2];
    frame[12]=value[3];

    //put CRC
    uint16 computedCrc;
    computedCrc = computeCrc(frame+1,1,12);
    computedCrc = ~computedCrc; // Invert the CRC

    frame[13] = computedCrc & 0xff; // Put the least sig. value in 1st byte transmitted.
    frame[14] = (computedCrc>>8) & 0xff;

    //check and replace special chars
    uchar frameWithoutSpecialChar[(2*length)+1];
    int newIndex=1;
    for(int index=1;index<length-1;index++)
    {
        if((frame[index]==0x7D)||(frame[index]==0xC0)||(frame[index]==0xC1))
        {
            frameWithoutSpecialChar[newIndex]=0x7D;
            newIndex++;
            frameWithoutSpecialChar[newIndex]=(frame[index] xor 0x20);
            newIndex++;
            length++;
        }
        else    //copy only
        {
            frameWithoutSpecialChar[newIndex]=frame[index];
            newIndex++;
        }
    }
    if(length!=16)  //copy if at least one special char
    {
        for(int index=1;index<length-1;index++) frame[index]=frameWithoutSpecialChar[index];
    }

    //put end
    frame[length-1]=END_OF_FRAME;
    frame[length]='\0'; //to close
}
*/
void SerialFrame::makeFrameAPI(QByteArray _data, uint16_t address_API)
{
    //create a frame with parameters elements
    erase();    //to be sure
    makeFrame(_data);

    for(int i=length-1;i>=0;i--)
        frame[i+8]=frame[i];
    length=length+9;
    frame[0]=START_OF_FRAME_API;
    frame[1]=0x00;
    frame[2]=length-4;
    frame[3]=0x01;
    frame[4]=0x00;
    placeUint16InUcharTab(address_API,frame,5);
    frame[7]=0x01;
    frame[length-1]=ComputeChecksum();
    frame[length]='\0'; //to close
}



void SerialFrame::makeFrameAPI868(QByteArray _data, uint64_t address_API)
{

    //create a frame with parameters elements
    erase();    //to be sure
    makeFrame(_data);
    length=length+15;

    for(int i=length;i>=0;i--)
        frame[i+14]=frame[i];
    frame[0]=START_OF_FRAME_API;
    frame[1]=0x00;
    frame[2]=length-4;
    frame[3]=0x00;
    frame[4]=0x00;
   // qDebug()<<address_API;
    placeUint64InUcharTab(address_API,frame,5);
    frame[13]=0x01;
    frame[length-1]=ComputeChecksum();
    frame[length]='\0'; //to close
}

void SerialFrame::makeframe_wdata(uint32 serialNb, uchar function,uchar param[17], uint16_t sn_relay){
    //create a frame with parameters elements
    erase();    //to be sure

    // Set API Bytes
    QByteArray _data;
      //put SN
    _data.append(hhh(serialNb));
    _data.append(hh(serialNb));
    _data.append(hi(serialNb));
    _data.append(lo(serialNb));

    //put function
    _data.append(function);
    uint8_t var_0=0;
    _data.append(var_0);
    //put value
    for(int i=0;i<17;i++){
        _data.append(param[i]);
    }

    makeFrame(_data);
    for(int i=length-1;i>=0;i--)
        frame[i+8]=frame[i];
    length=length+9;
    frame[0]=START_OF_FRAME_API;
    frame[1]=0x00;
    frame[2]=length-4;
    uint16_t serial_16=serialNb&0xFFFF;
    if(sn_relay!=0)
    { //passe par un relais
        frame[3]=0x51;
        placeUint16InUcharTab(serial_16,frame,4);
        placeUint16InUcharTab(sn_relay,frame,6);
    }
    else
    {
        frame[3]=0x01;
        frame[4]=0x00;
        placeUint16InUcharTab(serial_16,frame,5);
        frame[7]=0x01;
    }
    frame[length-1]=ComputeChecksum();
    frame[length]='\0'; //to close


    /*
    frame[0]=START_OF_FRAME_API;
    frame[1]=0x00;
    frame[2]=0x00;
    uint16_t serial_16=serialNb&0xFFFF;
    if(sn_relay!=0){ //passe par un relais
        frame[3]=0x51;
        placeUint16InUcharTab(serial_16,frame,4);
        placeUint16InUcharTab(sn_relay,frame,6);
    }
    else {
    frame[3]=0x01;
    frame[4]=0x00;

    placeUint16InUcharTab(serial_16,frame,5);
    frame[7]=0x01;
    }

    frame[0+8]=START_OF_FRAME;

    //put SN
    placeUint32InUcharTab(serialNb,frame,1+8);

    //put function
    frame[5+8]=function;
    frame[6+8]=0x00;
    //put value
    for(int i=0;i<17;i++){
        frame[15+i]=param[i];
    }

    //put CRC
    uint16 computedCrc;
    computedCrc = computeCrc(frame,9,20);
    computedCrc = ~computedCrc; // compute the CRC

    frame[29] = computedCrc & 0xff; // Put the least sig. value in 1st byte transmitted.
    frame[30] = (computedCrc>>8) & 0xff;

    //check special chars and get true length
     check_special_char_in_frame_to_send();
     frame[2]=length;
    //put end
    frame[length+7]=END_OF_FRAME;
    frame[length+8]=ComputeChecksum();
    frame[length+9]='\0'; //to close*/
}


void  SerialFrame::makeSimpleAPIFrame(QByteArray data_to_send){
    erase();
    length=data_to_send.size()+4;
    frame[0]=START_OF_FRAME_API;
    frame[1]=0x00;
    frame[2]=data_to_send.size();
    //put data
    for(int i=0;i<data_to_send.size();i++){
       frame[3+i]=data_to_send.at(i);
    }
    //put end
    frame[length-1]=ComputeChecksum();
    frame[length]='\0'; //to close
}

//**************************************
// Debug methods
//**************************************
void SerialFrame::display()
{
    //To display a frame
    QString debugLine = "frame ("+QString::number(length)+") : ";
    for(int i=0; i<length;i++) debugLine += QString::number(frame[i],16).rightJustified(2,'0')+" ";
    qDebug()<<debugLine;
}



uint16 SerialFrame::ComputeChecksum()
{
    uint16 cs=0x0000;
    //qDebug()<<"cs l"<<length;
    for(int index=3;index<(length-1);index++)
    {
        cs=cs+frame[index];
    }
    cs=0x00FF-(0x00FF & cs);
    return cs;
}

