#include "waveform.h"

waveform::waveform(ManageSerialPort *_serialPort,RefreshTimer *_commonRefreshTimer,Module *_power_module,int _current_channel,float _coeff_module, QWidget *parent) :
    QWidget(parent)
{
    plot=new Plot;
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/WAVEFORM_WIN_X")&&
            settingFile->contains("WINDOW_POS/WAVEFORM_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/WAVEFORM_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/WAVEFORM_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    ref_paused=false;
    coeff_i=_coeff_module/200.;
    i_scale_old=200.;
    currentChannel=_current_channel;
    power_module=_power_module;
    commonRefreshTimer=_commonRefreshTimer;
    setMinimumSize(600,400);
    nb_sample=117;
    size_of_u_sample=1;
    size_of_i_sample=2;
    coeff_u=2*1.414;
    for(uint16_t i=0;i<nb_sample;i++) x[i]=i*20./nb_sample;
    serialCom = _serialPort;
    setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/waveform.png"));
    setWindowTitle("Waveforms");
    setWindowModality(Qt::ApplicationModal);

    plot->setTitle("Waveforms Module "+QString::number(power_module->serialNb,16).rightJustified(8,'0')+"  Channel "+QString::number(currentChannel));
    QVBoxLayout *layoutv1 = new QVBoxLayout;
    box_layout_v2 = new QGroupBox;
    QVBoxLayout *layoutv2 = new QVBoxLayout;
    layoutv2->setContentsMargins( 0, 0, 0, 0 );
    layoutv2->addWidget( plot );
    QHBoxLayout *layouth1 = new QHBoxLayout;
    powerValue=new QLabel;
    apparentPowerValue=new QLabel;
    reactivePowerValue=new QLabel;
    voltageValue=new QLabel;
    currentValue=new QLabel;
    powerFactorValue=new QLabel;
    //QLabel *empty_widg=new QLabel;
    QVBoxLayout *layoutval1 = new QVBoxLayout;
    QVBoxLayout *layoutval2 = new QVBoxLayout;
    QVBoxLayout *layoutval3 = new QVBoxLayout;
    layoutval1->addWidget(voltageValue);
    layoutval1->addWidget(apparentPowerValue);
    layoutval2->addWidget(currentValue);
    layoutval2->addWidget(reactivePowerValue);
    layoutval3->addWidget(powerValue);
    layoutval3->addWidget(powerFactorValue);
    layouth1->addLayout(layoutval1);
    layouth1->addLayout(layoutval2);
    layouth1->addLayout(layoutval3);
    layoutv2->addLayout(layouth1);
    box_layout_v2->setLayout(layoutv2);
    layoutv1->addWidget(box_layout_v2);
    QHBoxLayout *layouth_chan = new QHBoxLayout;
    chan_b_group=new QButtonGroup;
    chan1 = new QRadioButton("Channel 1");
    chan2 = new QRadioButton("Channel 2");
    chan3 = new QRadioButton("Channel 3");
    chan4 = new QRadioButton("Channel 4");
    chan5 = new QRadioButton("Channel 5");
    chan6 = new QRadioButton("Channel 6");
    chan_b_group->addButton(chan1);
    chan_b_group->addButton(chan2);
    chan_b_group->addButton(chan3);
    chan_b_group->addButton(chan4);
    chan_b_group->addButton(chan5);
    chan_b_group->addButton(chan6);
    chan_b_group->setId(chan1,1);
    chan_b_group->setId(chan2,2);
    chan_b_group->setId(chan3,3);
    chan_b_group->setId(chan4,4);
    chan_b_group->setId(chan5,5);
    chan_b_group->setId(chan6,6);
    switch(currentChannel){
    case 1:
        chan1->setChecked(true);
        break;
    case 2:
        chan2->setChecked(true);
        break;
    case 3:
        chan3->setChecked(true);
        break;
    case 4:
        chan4->setChecked(true);
        break;
    case 5:
        chan5->setChecked(true);
        break;
    case 6:
        chan6->setChecked(true);
        break;
    }
    connect(chan_b_group,SIGNAL(buttonClicked(int)),this,SLOT(select_channel(int)));
    layouth_chan->addWidget(chan1);
    layouth_chan->addWidget(chan2);
    layouth_chan->addWidget(chan3);
    layouth_chan->addWidget(chan4);
    layouth_chan->addWidget(chan5);
    layouth_chan->addWidget(chan6);
    layoutv1->addLayout(layouth_chan);
    QHBoxLayout *layouth3 = new QHBoxLayout;
    export_as= new QPushButton("Export as image file");
    connect(export_as,SIGNAL(clicked()),this,SLOT(export_file()));
    layouth3->addWidget(export_as);
    play_pause= new QPushButton();
    play_pause->setIcon(QIcon(QString(DEFAULT_DIR)+"icon/pause.png"));
    play_pause->setText("&Pause Refresh");
    connect(play_pause,SIGNAL(clicked()),this,SLOT(switchPauseRefresh()));
    layouth3->addWidget(play_pause);
    layoutv1->addLayout(layouth3);
    setLayout(layoutv1);


    if(serialCom->writeByteAtWithRetry(power_module->serialNb,0x01D0,currentChannel-1,1));
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval();   //small delay before first refresh
    delete settingFile;
}

Plot::Plot(QWidget *parent):
    QwtPlot( parent )
{
    setAutoFillBackground( true );
    setPalette( QPalette( QColor( 165, 193, 228 ) ) );

    // axes
    setAxisTitle(xBottom, "Time (ms)" );
    setAxisScale(xBottom, 0., 20.);

    setAxisTitle(yLeft, "<font color= red> Voltage (V) </font>");
    enableAxis(QwtPlot::yRight);
    setAxisTitle(QwtPlot::yRight, "<font color= blue> Current (A) </font>");

    // canvas
    //canvas()->setLineWidth( 1 );
   // canvas()->setFrameStyle( QFrame::Box | QFrame::Plain );
  //  canvas()->setBorderRadius( 15 );

    QPalette canvasPalette( Qt::white );
    canvasPalette.setColor( QPalette::Foreground, QColor( 133, 190, 232 ) );
    canvas()->setPalette( canvasPalette );
    u_curve=new QwtPlotCurve("Voltage");
    u_curve->setRenderHint(QwtPlotItem::RenderAntialiased);
    u_curve->setLegendAttribute(QwtPlotCurve::LegendShowLine, true);
    u_curve->setPen(QPen(Qt::red));
    u_curve->setAxes(QwtPlot::xBottom, QwtPlot::yLeft);
    setAxisScale( QwtPlot::yLeft,  -400, 400 );
    u_curve->attach(this);


    i_curve=new QwtPlotCurve("Current");
    i_curve->setRenderHint(QwtPlotItem::RenderAntialiased);
    i_curve->setLegendAttribute(QwtPlotCurve::LegendShowLine, true);
    i_curve->setPen(QPen(Qt::blue));
    i_curve->setAxes(QwtPlot::xBottom, QwtPlot::yRight );
    setAxisScale( QwtPlot::yRight,  -100, 100 );
    i_curve->attach(this);

    QwtPlotMarker *mY0 = new QwtPlotMarker();
    mY0->setLabel(QString::fromLatin1("0"));
    mY0->setLabelAlignment(Qt::AlignRight|Qt::AlignTop);
    mY0->setLineStyle(QwtPlotMarker::HLine);
    mY0->setYValue(0.0);
    mY0->attach(this);
}

void waveform::timerSlot()
{
    if(ref_paused) return;
    //Connected to timer, Refresh waveform

    //Get the name of module, and display it
    QString chann_name;
    if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x010000A0+16*(currentChannel-1),2,3))
    {
        char channelName[17];
        for(int charIndex=0; charIndex<16; charIndex++)
        {
            if((serialCom->getReceivedData()[charIndex]>0x1F)&&(serialCom->getReceivedData()[charIndex]!=0xFF))
            {
                channelName[charIndex]=serialCom->getReceivedData()[charIndex];
            }
            else
            {
                channelName[charIndex]='\0';
            }
        }
        if(channelName[0]!='\0') chann_name=QString::fromStdString(channelName);
        else chann_name="Channel "+QString::number(currentChannel);
    }
    plot->setTitle("Waveforms Module "+QString::number(power_module->serialNb,16).rightJustified(8,'0')+", "+chann_name);
    update_waveform();
    update_value();



    if(serialCom->writeByteAtWithRetry(power_module->serialNb,0x01D0,currentChannel-1,1));
    commonRefreshTimer->startInterval();

}

void waveform::update_waveform(){

    uint16_t address=0x200;
    uint16_t i=0;
    uint16_t current_index=0;
    commonRefreshTimer->stop();
    //serialCom->deleteErrorCom();    //retry
    do{
        if(!serialCom->readBytesAtWithRetry(power_module->serialNb,address,8,3)) return;
        i+=64;
        address+=64;
        for(int j=0;j<64;j++){
            if(current_index<nb_sample) {
                if(size_of_u_sample==1) tab_voltage_temp[current_index]=peekInt8FromUcharTab(serialCom->getReceivedData(),j)*coeff_u;
                if(size_of_u_sample==2) tab_voltage_temp[current_index]=peekInt16FromUcharTabBigEndian(serialCom->getReceivedData(),j*2)*coeff_u;
                current_index++;
            }
        }
    } while(i<(nb_sample*size_of_u_sample));
    // Get current value
    address=0x300;
    current_index=0;
    i=0;
    float max_i=0;
    do{
        if(!serialCom->readBytesAtWithRetry(power_module->serialNb,address,8,3)) return;
        i+=64;
        address+=64;
        for(int j=0;j<(64/size_of_i_sample);j++){
            if(current_index<nb_sample) {
                tab_current_temp[current_index]=peekInt16FromUcharTabBigEndian(serialCom->getReceivedData(),j*size_of_i_sample)*coeff_i;
                if(fabs(tab_current_temp[current_index])>max_i) max_i=fabs(tab_current_temp[current_index]);
                current_index++;
            }
        }
    } while(i<(nb_sample*size_of_i_sample));

    // Look for zero cross.
   // if(power_module->typeOfModule==powerModule)
   // {
        uint16_t k=1;
        if(i>((nb_sample*size_of_u_sample)-1)){
            zero_cross_index=0;
            do
            {
                k++;
            }
            while(!((tab_voltage_temp[k-1]<=0)&&(tab_voltage_temp[k]>0))&&(k<nb_sample));
            zero_cross_index=k;
            for(int j=zero_cross_index;j<nb_sample;j++){
                tab_voltage[j-zero_cross_index]=tab_voltage_temp[j];
            }
            for(int j=0;j<zero_cross_index;j++){
                tab_voltage[nb_sample-zero_cross_index+j]=tab_voltage_temp[j];
            }
        }
        for(int j=zero_cross_index;j<nb_sample;j++){
            tab_current[j-zero_cross_index]=tab_current_temp[j];
        }
        for(int j=0;j<zero_cross_index;j++){
            tab_current[nb_sample-zero_cross_index+j]=tab_current_temp[j];
        }
    //}

    if(i>((nb_sample*size_of_u_sample)-1)){
        float i_scale;
        //float i_max_reel=max_i*coeff_i;
        if((max_i>1000)||((i_scale_old==2500.)&&(max_i>=800))) i_scale=2500.;
        else if((max_i>500)||((i_scale_old==1000.)&&(max_i>=450))) i_scale=1000.;
        else if((max_i>200)||((i_scale_old==500.)&&(max_i>=180))) i_scale=500.;
        else if((max_i>150)||((i_scale_old==200.)&&(max_i>=140))) i_scale=200.;
        else if((max_i>100)||((i_scale_old==150.)&&(max_i>=90))) i_scale=150.;
        else if((max_i>75)||((i_scale_old==100.)&&(max_i>=65))) i_scale=100.;
        else if((max_i>50)||((i_scale_old==75.)&&(max_i>=40))) i_scale=75.;
        else if((max_i>25)||((i_scale_old==50.)&&(max_i>=19))) i_scale=50.;
        else if((max_i>10.)||((i_scale_old==25.)&&(max_i>=8))) i_scale=25.;
        else if((max_i>5.)||((i_scale_old==10.)&&(max_i>=4.5))) i_scale=10.;
        else if((max_i>2.0)||((i_scale_old==5.)&&(max_i>=1.8))) i_scale=5.;
        else if((max_i>1.)||((i_scale_old==2.)&&(max_i>=0.8))) i_scale=2.;
        else if((max_i>0.5)||((i_scale_old==1.)&&(max_i>=0.4))) i_scale=1.;
        else i_scale=0.5;
        i_scale_old=i_scale;
        plot->setAxisScale( QwtPlot::yRight,  -i_scale, i_scale );



    }
    //if(power_module->typeOfModule==powerModule)
   // {
        plot->u_curve->setRawSamples(x,tab_voltage,nb_sample);
        plot->i_curve->setRawSamples(x,tab_current,nb_sample);
   // }

    /*else
    {
        plot->u_curve->setRawSamples(x,tab_voltage_temp,nb_sample);
        plot->i_curve->setRawSamples(x,tab_current_temp,nb_sample);
    }*/

    plot->QwtPlot::replot();


}

void waveform::update_value(){
    float power=0;
    float voltage=0;
    float current=0;
    float powerFactor=0;
    float apparent=0;

    // Power
    if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x1E0+2*(currentChannel-1),1,3))
    {
        power=int16(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),0));
        power= power*coeff_i*200;
        powerValue->setText("<b>Power:</b> "+QString::number(power,'f',0)+" <b>Watts</b>");
    }

    //Voltage
    if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x110+(currentChannel-1),1,3))
    {
    phase=serialCom->getReceivedData()[0] & 0x03;
    }
    if(serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x100+2*(phase & 0x03),1,3))
    {
        voltage =sqrt(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),0) / 5859.0 / 2.0) *256.0;
        voltageValue->setText("<b>Voltage RMS:</b> "+QString::number(voltage,'f',1)+" <b>Volts</b>");
    }

        //Current
    if(power_module->typeOfModule==powerModule){//ancien module

        if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x520+4*(currentChannel-1),1,3))
        {
            current=sqrt(peekUint32FromUcharTabBigEndian(serialCom->getReceivedData(),0) / 117.0 * 256.0 /8.0) /200.0;
            current=current * coeff_i *200;
            currentValue->setText("<b>Current RMS</b> :"+QString::number(current,'f',2)+" <b>A</b>");
        }
    }
    else
    {//nouveaux module (typeOfModule=powerModule2)
        if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x520+6*(currentChannel-1),1,3))
        {
            current=peekUint48FromUcharTabBigEndian(serialCom->getReceivedData(),0);
            current=sqrt(current)/19442.2;
            current=current * coeff_i *200;
            if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x110+(currentChannel-1),1,3))// check over-limit
            {
                if(serialCom->getReceivedData()[0]&0x20){
                   currentValue->setText("<b>Current RMS</b> : <font color= red> > "+QString::number(current,'f',2)+" </font><b>A</b>");
                }
                else currentValue->setText("<b>Current RMS</b> :"+QString::number(current,'f',2)+" <b>A</b>");
            }
        }
    }

    //Apparent Power
  /*  if(power<0) power=-power;
    float apparent=current*voltage;
    if(apparent < 5) apparent = 0;
    if(apparent < power) apparent = power;
    apparentPowerValue->setText("<b>Apparent Power:</b> "+QString::number(apparent,'f',0)+" <b>VA</b>");*/

    if(power_module->typeOfModule==powerModule)
    {
        //Apparent Power
        if(power<0) power=-power;
        apparent=current*voltage;
        if(apparent < 5) apparent = 0;
        if(apparent < power) apparent = power;
        apparentPowerValue->setText("<b>Apparent Power</b> :"+QString::number(apparent,'f',0)+" <b>VA</b>");

        //Reactive Power
        float reactif = sqrt(apparent*apparent-power*power);
        reactivePowerValue->setText("<b>Reactive Power</b> :"+QString::number(reactif,'f',0)+" <b>VAR</b>");
    }
    else
    {
        if(serialCom->readBytesAtWithRetry(power_module->serialNb,0x170+2*(currentChannel-1),1,3)){
            //Apparent Power
            float reactif=peekInt16FromUcharTabBigEndian(serialCom->getReceivedData(),0)* coeff_i *200;
            reactivePowerValue->setText("<b>Reactive Power</b> :"+QString::number(reactif,'f',0)+" <b>VAR</b>");
            //Reactive Power
            apparent=sqrt(reactif*reactif+power*power);
            apparentPowerValue->setText("<b>Apparent Power</b> :"+QString::number(apparent,'f',0)+" <b>VA</b>");
        }
    }
    //Power Factor
    if(apparent>0) powerFactor=power/apparent;
    else powerFactor=0;
    powerFactorValue->setText("<b>PF:</b> "+QString::number(powerFactor,'f',2));
}

void waveform::closeEvent(QCloseEvent *event)
 {
    commonRefreshTimer->stop();
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    settingFile->setValue("WINDOW_POS/WAVEFORM_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/WAVEFORM_WIN_Y",QVariant(QString::number(this->pos().y())));
    delete settingFile;

    setVisible(false);
    emit waveformClosed();
}
/*
void waveform::resizeEvent( QResizeEvent *event )
{
    plot->QwtPlot::resizeEvent( event );
}
*/

void waveform::export_file()
{ QString fileName;

    fileName = QFileDialog::getSaveFileName(this, tr("Nom du fichier"), QString(),"Graphic files (*.png )");

    if ( !fileName.isEmpty() )
    {


    QPixmap pixmap= QPixmap::grabWidget(box_layout_v2);

    if ( pixmap.save(fileName, "png" ))
    qDebug()<<"ok";
    else
    qDebug()<<"Uhmm";
    }
}
void waveform::switchPauseRefresh()
{

    if(ref_paused)
    {
        //un-pause
        commonRefreshTimer->restartAfterPause();
        ref_paused=false;
        play_pause->setIcon(QIcon(QString(DEFAULT_DIR)+"icon/pause.png"));
        play_pause->setText("&Pause Refresh");

        //enable window

    }
    else    //refresh not paused
    {

        //pause
        commonRefreshTimer->stop();
        ref_paused=true;
        play_pause->setIcon(QIcon(QString(DEFAULT_DIR)+"icon/play.png"));
        play_pause->setText("&Refresh Again");

        //disable window


    }
}

void waveform::select_channel(int channel){
    currentChannel=channel;
    if(serialCom->writeByteAtWithRetry(power_module->serialNb,0x01D0,currentChannel-1,1));
}
