#include "wirelessmodule2.h"

WirelessModule2::WirelessModule2(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, Module *_wireless_Module, bool _advancedUserMode, QWidget *parent) :
    QWidget(parent)
{
    //Constructor

    //init values
    serialCom=_serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    wireless_Module=_wireless_Module;
    firstRefresh=false;
    fillingSensorTable=false;
    refresh_active=false;
    advancedUserMode=_advancedUserMode;
    signalMapper = new QSignalMapper(this);
    batt_volt<<"~"<<"2.9V"<<"3.0V"<<"3.4V"<<"2.2V"<<"2.3V"<<"2.4V"<<
        "2.5V"<<"2.6V"<<"2.7V"<<"2.8V"<<"3.1V"<<"3.2V"<<"3.3V"<<"3.5V"<<"3.7V";

    batt_volt2<<"~"<<"2.9"<<"3.0"<<"3.4"<<"2.2"<<"2.3"<<"2.4"<<
        "2.5"<<"2.6"<<"2.7"<<"2.8"<<"3.1"<<"3.2"<<"3.3"<<"3.5"<<"3.7";
/*
    // Init Channel type Lists
    listTempHygroPulse <<"Temperature"<<"Humidity"<<"Pulse Count"<<"ON Time";
    listTempPulse <<"Temperature"<<"Average Temp."<<"Pulse Count"<<"ON Time";
    listLamp <<"Light Sensor"<<"Switch Count"<<"ON Time";
    listWattmeter <<"Volts"<<"Amps"<<"Active Power"<<"Apparent Power"
                  <<"Active Energy"<<"Appar. Energy";
    batt_volt<<"~"<<"2.9V"<<"3.0V"<<"3.4V"<<"2.2V"<<"2.3V"<<"2.4V"<<
        "2.5V"<<"2.6V"<<"2.7V"<<"2.8V"<<"3.1V"<<"3.2V"<<"3.3V"<<"3.5V"<<"3.7V";
    // To map signals from channel type comboBoxes
    comboBoxSignalMapper = new QSignalMapper(this);*/

    //Add GUI elements
    pointeur_list_sensor_w=0;
    fillWindow();

    //put columns
    setTableWidget();

    add_empty_row();
    connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(select_changed(int)));
    refresh_values();
    //start timer
   // commonRefreshTimer->connectTimeout(this);
  //  firstRefresh=true;
  //  commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

void WirelessModule2::fillWindow()
{
    //Add GUI elements
    globalLayout = new QVBoxLayout;
    setLayout(globalLayout);
        groupBoxWireless=new QGroupBox("Wireless sensors");
            verticalLayout = new QVBoxLayout;
            groupBoxWireless->setLayout(verticalLayout);
                sensorTableWidget = new QTableWidget(0,6);
                verticalLayout->addWidget(sensorTableWidget);


                buttonLayout = new QHBoxLayout;
                start_campaign= new QPushButton("Start Campaign");
                buttonLayout->addWidget(start_campaign);
                connect(start_campaign,SIGNAL(clicked()),this,SLOT(start_campaign_clicked()));
                refresh_list= new QPushButton("Refresh List");
                buttonLayout->addWidget(refresh_list);
                connect(refresh_list,SIGNAL(clicked()),this,SLOT(refresh_values()));
                verticalLayout->addLayout(buttonLayout);

                if(advancedUserMode)
                {
                    QHBoxLayout *set_start_date = new QHBoxLayout;
                    QLabel *date_start_l = new QLabel("Start date");
                    date_start_l->setFixedWidth(60);
                    start_date_sensor = new QDateTimeEdit(QDateTime::currentDateTime());
                    start_date_sensor->setCalendarPopup(true);
                    start_date_sensor->setMinimumDateTime(QDateTime::fromTime_t(0));
                    start_date_sensor->setMaximumDateTime(QDateTime::currentDateTime().addSecs(7200));
                    start_date_sensor->setDisplayFormat("dd/MM/yyyy hh:mm:ss");
                    start_date_sensor->dateTime().setTimeSpec(Qt::LocalTime);
                    set_date = new QPushButton("Set start Date");
                    set_date->setFixedWidth(120);
                    set_start_date->addWidget(date_start_l);
                    set_start_date->addWidget(start_date_sensor);
                    set_start_date->addWidget(set_date);
                    set_start_date->setAlignment(Qt::AlignLeft);
                    connect(set_date,SIGNAL(clicked()),this,SLOT(set_start_clicked()));
                    verticalLayout->addLayout(set_start_date);


                    QHBoxLayout *adv_vidage_layout = new QHBoxLayout;
                    QLabel *date_vidage_l = new QLabel("End date");
                    date_vidage_l->setFixedWidth(60);
                    date_vidage_sensor = new QDateTimeEdit(QDateTime::currentDateTime());
                    date_vidage_sensor->setCalendarPopup(true);
                    date_vidage_sensor->setMinimumDateTime(QDateTime::fromTime_t(0));
                    date_vidage_sensor->setMaximumDateTime(QDateTime::currentDateTime().addSecs(7200));
                    date_vidage_sensor->setDisplayFormat("dd/MM/yyyy hh:mm:ss");
                    date_vidage_sensor->dateTime().setTimeSpec(Qt::LocalTime);
                    start_vidage = new QPushButton("Start Dump");
                    start_vidage->setFixedWidth(120);
                    get_file = new QPushButton("Save data");
                    get_file->setFixedWidth(120);
                    adv_vidage_layout->addWidget(date_vidage_l);
                    adv_vidage_layout->addWidget(date_vidage_sensor);
                    adv_vidage_layout->addWidget(start_vidage);
                    adv_vidage_layout->addWidget(get_file);
                    adv_vidage_layout->setAlignment(Qt::AlignLeft);
                    connect(start_vidage,SIGNAL(clicked()),this,SLOT(dump_clicked()));
                    connect(get_file,SIGNAL(clicked()),this,SLOT(get_data_clicked()));

                    verticalLayout->addLayout(adv_vidage_layout);
                }
                buttonLayout2 = new QHBoxLayout;
                    returnButton = new QPushButton("<= Back");
                    buttonLayout2->addWidget(returnButton);
                    connect(returnButton,SIGNAL(clicked()),this,SLOT(backToManageModule()));
                    verticalLayout->addLayout(buttonLayout2);
                    //returnButton->setEnabled(false);

                    //emptyButton->setEnabled(false);

    globalLayout->addWidget(groupBoxWireless);
}

void WirelessModule2::setTableWidget()
{
    // Place columns into sensorTableWidget
    sensorTableWidget->setVisible(true);

    sensorSNTableItem = new QTableWidgetItem("SN");
    sensorTableWidget->setHorizontalHeaderItem(0,sensorSNTableItem);
    sensorTableWidget->setColumnWidth(0,60);

    sensorRelayTableItem = new QTableWidgetItem("Relay SN");
    sensorTableWidget->setHorizontalHeaderItem(1,sensorRelayTableItem);
    sensorTableWidget->setColumnWidth(1,60);

    sensorRSSITableItem = new QTableWidgetItem("RSSI");
    sensorTableWidget->setHorizontalHeaderItem(2,sensorRSSITableItem);
    sensorTableWidget->setColumnWidth(2,60);

    sensorDateVTableItem = new QTableWidgetItem("Date");
    sensorTableWidget->setHorizontalHeaderItem(3,sensorDateVTableItem);
    sensorTableWidget->setColumnWidth(3,120);

    sensorLastComTableItem = new QTableWidgetItem("Com");
    sensorTableWidget->setHorizontalHeaderItem(4,sensorLastComTableItem);
    sensorTableWidget->setColumnWidth(4,50);

    sensorSelectTableItem = new QTableWidgetItem("Select");
    sensorTableWidget->setHorizontalHeaderItem(5,sensorSelectTableItem);
    sensorTableWidget->setColumnWidth(5,40);

    sensorTableWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
    sensorTableWidget->setAutoScroll(false);

    sensorTableWidget->clearContents();
    sensorTableWidget->setRowCount(0);

    //connect to apply changes in a cell
    connect(sensorTableWidget,SIGNAL(cellChanged(int,int)),this,SLOT(valueChanged(int,int)));
}


void WirelessModule2::add_empty_row(){

    uint16_t n_row = sensorTableWidget->rowCount();
    sensorTableWidget->setRowCount(n_row+1);
    uint16_t row_index=n_row;
    //QTableWidgetItem *newLine [256];
    sensor_w current_sensor;
    current_sensor.serialNb=0;
    current_sensor.relay=0;
    current_sensor.date_dernier_vidage.setTime_t(0);
    current_sensor.rssi=255;
    current_sensor.last_com=255;
    list_sensor_w[pointeur_list_sensor_w++]=current_sensor;

    fillingSensorTable=true;
    // Add empty items, from 0 to 11
    for(uint16 columnIndex=0; columnIndex<6; columnIndex++)
    {
        //create item
        QTableWidgetItem *newItem = new QTableWidgetItem;
        newItem->setTextAlignment(Qt::AlignCenter|Qt::AlignVCenter);
        if(columnIndex!=0){ newItem->setFlags(newItem->flags()
                                & ~Qt::ItemIsEditable & ~Qt::ItemIsSelectable);
            newItem->setText("");
        }
        else {
            newItem->setFlags(newItem->flags() & ~Qt::ItemIsSelectable);
                                        // to let serialNb & sensorName editable
            newItem->setText("");
        }
        sensorItemTable[row_index][columnIndex]=newItem;
        //link table entry with widget :
        sensorTableWidget->setItem(row_index,columnIndex,newItem);
    }

    QCheckBox *select_s=new QCheckBox;
    select_sensor[row_index]=select_s;
    QFrame *frame1=new QFrame;
    QHBoxLayout *select_s_layout = new QHBoxLayout;
    select_s_layout->addWidget(select_s);
    select_s_layout->setMargin(2);
    select_s_layout->insertSpacing( 0,5 );
    frame1->setLayout(select_s_layout);
    select_sensor[row_index]->setEnabled(false);
    connect( select_sensor[row_index], SIGNAL(clicked()), signalMapper, SLOT(map()));
    signalMapper->setMapping(select_sensor[row_index], row_index);
    sensorTableWidget->setCellWidget(row_index,5,frame1);




    for(int i=0;i<n_row;i++)
    {
        for(uint16 columnIndex=0; columnIndex<6; columnIndex++)
        {
            sensorItemTable[i][columnIndex]->setFlags( sensorItemTable[i][columnIndex]->flags()
                                                               |Qt::ItemIsSelectable);
        }
    }
    sensorTableWidget->setRowHeight(row_index,20);
    fillingSensorTable=false;

}

void WirelessModule2::select_changed(int row)
{
    fillingSensorTable=true;
    qDebug()<<"row selected"<<row<<select_sensor[row]->isChecked();
    serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x0100008d,1,3); //recupere le nombre de capteur dans eep
    uint8_t n_sensor_eep=serialCom->getReceivedData()[0];
    if(select_sensor[row]->isChecked())
    {
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a0+n_sensor_eep*2,hi(list_sensor_w[row].serialNb),3); //ecrit capteur dans eep
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a1+n_sensor_eep*2,lo(list_sensor_w[row].serialNb),3);
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x0100008d,n_sensor_eep+1,3);
    }
    else
    {
        uint8_t pos_sensor=0;
        serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x010000a0,8,3); //recuper la liste des capteur dans eep
        for(int i=0;i<n_sensor_eep;i++)
        {
            uint16_t current_sensor_eep=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),i*2);
            if(current_sensor_eep==list_sensor_w[row].serialNb)
            {
                pos_sensor=i;
                break;
            }
        }
        uint16_t serial_last_sensor=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),(n_sensor_eep-1)*2);
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a0+pos_sensor*2,hi(serial_last_sensor),3); //remplace capteur  supprimer par dernier capteur de la liste
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a1+pos_sensor*2,lo(serial_last_sensor),3);
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a0+(n_sensor_eep-1)*2,0,3); //met 0x0000 en fin de liste
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000a1+(n_sensor_eep-1)*2,0,3);
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x0100008d,n_sensor_eep-1,3);
    }
    fillingSensorTable=false;
}

void WirelessModule2::start_campaign_clicked()
{
    fillingSensorTable=true;
    QDateTime dl_start_t;
    dl_start_t=dl_start_t.toLocalTime();
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x156,4,3); //clear list
    waitFor(100);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x306,bcdFromDecimal(dl_start_t.currentDateTime().date().year()%100),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x307,bcdFromDecimal(dl_start_t.currentDateTime().date().month()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x308,bcdFromDecimal(dl_start_t.currentDateTime().date().day()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x309,bcdFromDecimal(dl_start_t.currentDateTime().time().hour()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30a,bcdFromDecimal(dl_start_t.currentDateTime().time().minute()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30b,bcdFromDecimal(dl_start_t.currentDateTime().time().second()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x156,2,3); //set start time
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000020,0,3); //clear pointer flash
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000021,0,3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000022,0,3);
    waitFor(100);
    fillingSensorTable=false;
    refresh_values();
}

void WirelessModule2::valueChanged(int row, int column)
{
    //A cell text changed (serial Nb or name)
    if((column<12) && !fillingSensorTable) //To avoid signal from GUI
    {
        switch(column)
        {
        case 0:
            setSerialNb(row);
            break;
        default:
            //qDebug()<<"Value changed in unauthorized cell !";
            break;
        }
    }
}

void WirelessModule2::setSerialNb(int _row){
    //To change serial number
    bool ok=true;
    fillingSensorTable=true;
    int max_line;
    max_line=sensorTableWidget->rowCount()-1;
    uint16_t newSerialNb=sensorItemTable[_row][0]->text().toInt(&ok,16);
    qDebug()<<"serial"<<QString::number(newSerialNb,16).rightJustified(4,'0').toUpper();
    sensorItemTable[_row][0]->text().toInt(&ok,10);
    if(!ok||(newSerialNb<=0))
    {
        QMessageBox::critical(this,"Error SN","Serial Number must be a strictly positive integer");
      if(_row<max_line)
          sensorItemTable[_row][0]->setText(QString::number(list_sensor_w[_row].serialNb,16).rightJustified(4,'0').toUpper());
       else sensorItemTable[_row][0]->setText("");
       fillingSensorTable=false;
       return;
    }
    bool sensor_exist=false;
    for(int i=0;i<pointeur_list_sensor_w;i++) if(list_sensor_w[i].serialNb==newSerialNb)sensor_exist=true;
    if(sensor_exist){
        if(_row<max_line)
        sensorItemTable[_row][0]->setText(QString::number(list_sensor_w[_row].serialNb,16).rightJustified(4,'0').toUpper());
        else sensorItemTable[_row][0]->setText("");
        fillingSensorTable=false;
        return;
    }

    uint8_t n_line_to_insert;
    if(_row<max_line) {
        delete_row(_row);
    }
    max_line=sensorTableWidget->rowCount()-1;
    int i=0;
    while((newSerialNb>list_sensor_w[i].serialNb)&&(i<(pointeur_list_sensor_w-1))){
        i++;
    }
    n_line_to_insert=i;
    qDebug()<<"n_line to insert"<<n_line_to_insert;
    for(int i=max_line-1;i>=n_line_to_insert;i--){
        //copy_sensor(sensorList[i+1],sensorList[i]);
        list_sensor_w[i+1]=list_sensor_w[i];
       // list_sensor_w[i+1].date_dernier_vidage=list_sensor_w[i].date_dernier_vidage;
        select_sensor[i+1]->setChecked(select_sensor[i]->isChecked());
        sensorItemTable[i+1][0]->setText(sensorItemTable[i][0]->text());
        sensorItemTable[i+1][1]->setText(sensorItemTable[i][1]->text());
        sensorItemTable[i+1][2]->setText(sensorItemTable[i][2]->text());
        sensorItemTable[i+1][3]->setText(sensorItemTable[i][3]->text());
        sensorItemTable[i+1][4]->setText(sensorItemTable[i][4]->text());
        select_sensor[i+1]->setEnabled(select_sensor[i]->isEnabled());
        select_sensor[i+1]->setChecked(select_sensor[i]->isChecked());
    }
    if(!refresh_active)
    {
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x300,hi(newSerialNb),3); //new serial
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x301,lo(newSerialNb),3); //new serial
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x156,0x10,3); //set new serial
    }
    list_sensor_w[n_line_to_insert].serialNb=newSerialNb;
    list_sensor_w[n_line_to_insert].date_dernier_vidage.setTime_t(0);
    list_sensor_w[n_line_to_insert].relay=0;
    list_sensor_w[n_line_to_insert].rssi=255;
    list_sensor_w[n_line_to_insert].last_com=255;

    sensorItemTable[n_line_to_insert][0]->setText(QString::number(newSerialNb,16).rightJustified(4,'0').toUpper());

    select_sensor[n_line_to_insert]->setEnabled(true);
    sensorItemTable[n_line_to_insert][1]->setText("");
    sensorItemTable[n_line_to_insert][2]->setText("");
    sensorItemTable[n_line_to_insert][4]->setText("");
    add_empty_row();
    fillingSensorTable=false;
    if(!refresh_active) refresh_values();

}

void WirelessModule2::delete_row(int row){
    fillingSensorTable=true;
    if(row<sensorTableWidget->rowCount()){
        int max_line;
        max_line=sensorTableWidget->rowCount()-1;
        for(int i=row;i<max_line;i++){
            //copy_sensor(sensorList[i],sensorList[i+1]);
            list_sensor_w[i]=list_sensor_w[i+1];
            select_sensor[i]->setChecked(select_sensor[i+1]->isChecked());
            sensorItemTable[i][0]->setText(sensorItemTable[i+1][0]->text());
            //sensorItemTable[i][0]->setTextColor(sensorItemTable[i+1][0]->textColor());
            sensorItemTable[i][1]->setText(sensorItemTable[i+1][1]->text());
            //sensorItemTable[i][1]->setTextColor(sensorItemTable[i+1][1]->textColor());

            sensorItemTable[i][2]->setText(sensorItemTable[i+1][2]->text());
           // sensorItemTable[i][2]->setTextColor(sensorItemTable[i+1][2]->textColor());
            sensorItemTable[i][3]->setText(sensorItemTable[i+1][3]->text());
            //sensorItemTable[i][3]->setTextColor(sensorItemTable[i+1][3]->textColor());
            sensorItemTable[i][4]->setText(sensorItemTable[i+1][4]->text());
           // sensorItemTable[i][4]->setTextColor(sensorItemTable[i+1][4]->textColor());
            select_sensor[i]->setEnabled(select_sensor[i+1]->isEnabled());
            select_sensor[i]->setChecked(select_sensor[i+1]->isChecked());


        }
       sensorTableWidget->removeRow(max_line);
       for(int i=0;i<11;i++)sensorItemTable[max_line-1][i]->setFlags(sensorItemTable[max_line-1][i]->flags()&~Qt::ItemIsSelectable);
    }
    pointeur_list_sensor_w--;
}
void WirelessModule2::setadvance()
{
    advancedUserMode=true;
}

void WirelessModule2::refresh_values()
{
    refresh_active=true;
    if(!serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x00000156,1,3))
        return; //demande la liste des capteurs
    waitFor(300);
    if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x00000500,64,3)) return; //recupere la liste des sn
    sensor_w list_temp[32];
    uint8_t list_temp_size=0;
    for(int i=0;i<32;i++)
    {
        if(!peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),i*2)) break;
        else list_temp[list_temp_size++].serialNb=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),i*2);
    }
    if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x00000540,8,3)) return; //recupere la liste des RSSI
    for(int i=0;i<list_temp_size;i++)
    {
        list_temp[i].rssi=serialCom->getReceivedData()[i];
    }
    serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x00000560,8,3); //recupere la liste des last com
    for(int i=0;i<list_temp_size;i++)
    {
        list_temp[i].last_com=serialCom->getReceivedData()[i];
    }

    if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x00000110,8,3)) return; //recupere la liste des last com
    for(int i=0;i<list_temp_size;i++)
    {
        list_temp[i].relay=peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),i*2);
    }

    uint8_t date_tab[256];
    uint8_t pointeur_date_tab=0;
    for(int k=0;k<3;k++)
    {
        if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x00000200+k*64,8,3)) return; //recupere les dates start
        for(int i=0;i<64;i++)
            date_tab[pointeur_date_tab++]=serialCom->getReceivedData()[i];
    }
    for(int i=0;i<list_temp_size;i++)
    {
        QDate rec_cur_date(QString::number(date_tab[0+i*6],16).toInt()+2000,QString::number(date_tab[1+i*6],16).toInt(),QString::number(date_tab[2+i*6],16).toInt());
        QTime rec_cur_time(QString::number(date_tab[3+i*6],16).toInt(),QString::number(date_tab[4+i*6],16).toInt(),QString::number(date_tab[5+i*6],16).toInt());
        QDateTime rec_cur_date_time(rec_cur_date,rec_cur_time,Qt::LocalTime);
        list_temp[i].date_dernier_vidage=rec_cur_date_time;
    }

    for(int i=0;i<list_temp_size;i++)
    {
        sensorItemTable[sensorTableWidget->rowCount()-1][0]->setText(QString::number(list_temp[i].serialNb,16).rightJustified(4,'0').toUpper());
        QApplication::processEvents();
        waitFor(1);
        while(fillingSensorTable);
        uint8_t row=0;
        //bool sensor_exist;
        for(int k=0;k<pointeur_list_sensor_w;k++)
        {
            if(list_sensor_w[k].serialNb==list_temp[i].serialNb)
            {
                row=k;
                qDebug()<<"row"<<row;
                break;
            }
        }
        list_sensor_w[row].relay=list_temp[i].relay;
        sensorItemTable[row][1]->setText(QString::number(list_sensor_w[row].relay,16).rightJustified(4,'0').toUpper());
        list_sensor_w[row].rssi=list_temp[i].rssi;
        sensorItemTable[row][2]->setText("- "+QString::number(list_sensor_w[row].rssi)+" dbm");
        list_sensor_w[row].last_com=list_temp[i].last_com;
        sensorItemTable[row][4]->setText(QString::number(list_sensor_w[row].last_com*4)+" s");
        list_sensor_w[row].date_dernier_vidage=list_temp[i].date_dernier_vidage;
        sensorItemTable[row][3]->setText(list_sensor_w[row].date_dernier_vidage.toString("dd/MM/yyyy  hh:mm:ss"));
    }
    if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x0100008d,1,3)) return; //recupere le nombre de capteur dans eep
    uint8_t n_sensor_eep=serialCom->getReceivedData()[0];
    if(!serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x010000a0,8,3)) return; //recuper la liste des capteur dans eep
    for(int i=0;i<n_sensor_eep;i++)
    {
        uint16_t current_sensor_eep=peekUint16FromUcharTabLittleEndian(serialCom->getReceivedData(),i*2);
        for(int j=0;j<pointeur_list_sensor_w;j++)
        {
            if(current_sensor_eep==list_sensor_w[j].serialNb)
            {

                select_sensor[j]->setChecked(true);
            }
        }

    }
    refresh_active=false;
}

bool WirelessModule2::openFile(uint8_t row){
    bool ok=true;

//*****************
//    Open file :
    uint16_t sensor_nr=list_sensor_vidage[row];
    QDateTime currentDateTime= QDateTime::currentDateTime();
    QString suggestedName=QString::number(sensor_nr,16).rightJustified(4,'0').toUpper()+"_"+currentDateTime.toString("ddMMyy_hhmmss")+".tsv";
    QString pathOfFile=/*folder_for_save.absolutePath()+"/"+*/suggestedName;
    tsvFile[row].setFileName ( pathOfFile);
    if(!tsvFile[row].open(QFile::WriteOnly | QFile::Truncate))
    {
        ok=false;
        QMessageBox::critical(this,"File Access Error","The file "+pathOfFile+" can't be created, please verify folder settings");
        return false;

    }
    tsvFile[row].flush();   //erase file
    tsvFileStream[row].setDevice(&tsvFile[row]);   //set target for stream*/
    return true;

}


bool WirelessModule2::updateFile()
{
//*********************
//    Header lines
    uint16_t sensor_nr=peekUint16FromUcharTabLittleEndian(buffer_page,0);
    uint8_t sensor_type=buffer_page[15]&0x1F;
    bool sensor_exist=false;
    int row=0;
    for(int i=0;i<pointeur_list_sensor_vidage;i++)
    {
        if(list_sensor_vidage[i]==sensor_nr)
        {
            sensor_exist=true;
            row=i;
            break;
        }
    }
    if(!sensor_exist)
    {
        qDebug()<<"serial :"<<sensor_nr;
        row=pointeur_list_sensor_vidage;
       list_sensor_vidage[pointeur_list_sensor_vidage++]=sensor_nr;

        //open file and Write  header :
        if(!openFile(row)){
            return false;
        }

        QString headerLine="Sensor nr : "+QString::number(sensor_nr,16).rightJustified(4,'0').toUpper();
        headerLine+="  Version "+QString::number(buffer_page[15]>>5);
        headerLine+="  Type "+QString::number(sensor_type);
        uint8_t winter_time=buffer_page[7]>>7;
        if(winter_time) headerLine+="  Winter Time";
        else headerLine+="  Summer Time";
        tsvFileStream[row]<<headerLine;
        headerLine.clear();

        headerLine="\r\nTime";

        switch(sensor_type){
        case 1:
            headerLine+="\tTemp\tTemp2\tPulses\tPulses2";
            break;
        case 2:
            headerLine+="\tTemp\tHygro\tPulses";
            break;
        case 3:
            headerLine+="\tLight\tSw ON\tTime ON";
            break;
        }
        headerLine+="\tBattvolt\tPulsesHi\tPulses2Hi";
        tsvFileStream[row]<<headerLine;
        headerLine.clear();
    }

//-------------------
//  Write samples loop
    QString lineToWrite;
    uint8_t nb_data_per_ech_max=0;
    switch(sensor_type){
    case 1:
        nb_data_per_ech_max=4;
        break;
    case 2:
        nb_data_per_ech_max=3;
        break;
    case 3:
        nb_data_per_ech_max=3;
        break;

    }
    //get date in beginning of page
    QDate dl_date(QString::number(buffer_page[2],16).toInt()+2000,QString::number(buffer_page[3],16).toInt(),QString::number(buffer_page[4],16).toInt());
    //get time
    QTime dl_time(QString::number(buffer_page[5],16).toInt(),QString::number(buffer_page[6],16).toInt(),QString::number(buffer_page[7]&0x7F,16).toInt());
    QDateTime dl_date_time(dl_date,dl_time,Qt::LocalTime);//------------

    uint8_t n_periode_dl=buffer_page[8]&0x0f;
    uint8_t data_to_rec_page=buffer_page[9];
    uint8_t n_data_per_ech=((data_to_rec_page&0x80)>>7)+((data_to_rec_page&0x40)>>6)+((data_to_rec_page&0x20)>>5)+((data_to_rec_page&0x10)>>4)
            +((data_to_rec_page&0x08)>>3)+((data_to_rec_page&0x04)>>2)+((data_to_rec_page&0x02)>>1)+(data_to_rec_page&0x01);
    uint16_t n_ech_tot=buffer_page[10];
    uint16_t pulses_hi=(buffer_page[11]<<8)+buffer_page[12];
    uint16_t pulses2_hi=(buffer_page[13]<<8)+buffer_page[14];
    uint8_t battvolt=buffer_page[8]>>4;
    uint16_t pulses_old=0;
    uint16_t pulses2_old=0;
    for(int j=0;j<n_ech_tot;j++)
    { //each sample
        lineToWrite="\r\n"+dl_date_time.toString("dd/MM/yy hh:mm:ss")+"\t"; //add date
        //qDebug()<<i<<" : "<<lineToWrite;
        long long value;
        int l=0;
        for(int k=0;k<nb_data_per_ech_max;k++){ //manage data in 1 sample
            if(k==0){
                switch(sensor_type){
                case 1:
                    if(data_to_rec_page&0x01){
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        l++;
                        if(value>32767)value=(value-65536);//pour grer les valeurs ngatives
                        lineToWrite+=QString::number(float(value)/100,'f',1); //temperature
                        lineToWrite+="\t";
                    }
                    else lineToWrite+="\t";
                    break;
                case 2:
                    if(data_to_rec_page&0x01) {
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        l++;
                        if(value>32767)value=(value-65536);//pour grer les valeurs ngatives
                        lineToWrite+=QString::number(float(value)/100,'f',1); //temperature
                        lineToWrite+="\t";
                    }
                    else lineToWrite+="\t";
                    break;
                case 3:
                    if(data_to_rec_page&0x01){
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        l++;
                        lineToWrite+=QString::number(float(value)/255*100,'f',0)+"\t"; //light
                    }
                    else lineToWrite+="\t";
                    break;
                }
            }
            if(k==1) {

                switch(sensor_type){
                case 1:
                    if(data_to_rec_page&0x02){
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        l++;
                        if(value>32767)value=(value-65536);//pour grer les valeurs ngatives
                        lineToWrite+=QString::number(float(value)/100,'f',1); //temperature 2
                        lineToWrite+="\t";
                    }
                    else lineToWrite+="\t";
                    break;
                case 2:
                    if(data_to_rec_page&0x02){
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        l++;
                        lineToWrite+=QString::number(float(value)/100,'f',1)+"\t"; //Hygro
                    }
                    else lineToWrite+="\t";
                    break;
                case 3:
                    if(data_to_rec_page&0x04) {
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        if(value<pulses_old) pulses_hi++;
                        pulses_old=value;
                        value+=(pulses_hi<<16);
                        l++;
                        lineToWrite+=QString::number(value,10)+"\t"; //Sw_ON
                    }
                    else lineToWrite+="\t";
                    break;
                }
            }

            if(k==2) {
                switch(sensor_type){
                case 1:
                    if(data_to_rec_page&0x04) {
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        if(value<pulses_old) pulses_hi++;
                        pulses_old=value;
                        value+=(pulses_hi<<16);
                        l++;
                        lineToWrite+=QString::number(value,10)+"\t"; //pulses
                    }
                    else lineToWrite+="\t";
                    break;

                case 2:
                    if(data_to_rec_page&0x04) {
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        if(value<pulses_old) pulses_hi++;
                        pulses_old=value;
                        value+=(pulses_hi<<16);
                        l++;
                        lineToWrite+=QString::number(value,10)+"\t"; //pulses
                    }
                    else lineToWrite+="\t";
                    break;

                case 3:
                    if(data_to_rec_page&0x08){
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        if(value<pulses2_old) pulses2_hi++;
                        pulses2_old=value;
                        value+=(pulses2_hi<<16);
                        l++;
                        lineToWrite+=QString::number(value,10)+"\t"; //Time_ON
                    }
                    else lineToWrite+="\t";
                    break;
                }
            }

            if(k==3){
                switch(sensor_type){
                case 1:
                    if(data_to_rec_page&0x08) {
                        value=(buffer_page[16+j*n_data_per_ech*2+l*2+1]<<8)+buffer_page[16+j*n_data_per_ech*2+l*2];
                        if(value<pulses2_old) pulses2_hi++;
                        pulses2_old=value;
                        value+=(pulses2_hi<<16);
                        l++;
                        lineToWrite+=QString::number(value,10)+"\t"; //pulses 2
                    }
                    else lineToWrite+="\t";
                    break;
                }
            }
        }
        if(battvolt<16)
            lineToWrite+=batt_volt2.at(battvolt)+"\t"; //debug//------------------------------------------
        else lineToWrite+="~\t";                   //debug//------------------------------------------
        lineToWrite+=QString::number(pulses_hi,10)+"\t"; //debug//------------------------------------------
        lineToWrite+=QString::number(pulses2_hi,10)+"\t"; //debug//------------------------------------------
        tsvFileStream[row]<<lineToWrite;
        //lineToWrite.clear();
        switch(n_periode_dl){ //calculate the next date
        case 0:
            dl_date_time=dl_date_time.addSecs(1);
            break;
        case 1:
            dl_date_time=dl_date_time.addSecs(2);
            break;
        case 2:
            dl_date_time=dl_date_time.addSecs(5);
            break;
        case 3:
            dl_date_time=dl_date_time.addSecs(10);
            break;
        case 4:
            dl_date_time=dl_date_time.addSecs(15);
            break;
        case 5:
            dl_date_time=dl_date_time.addSecs(20);
            break;
        case 6:
            dl_date_time=dl_date_time.addSecs(30);
            break;
        case 7:
            dl_date_time=dl_date_time.addSecs(60);
            break;
        case 8:
            dl_date_time=dl_date_time.addSecs(600);
            break;
        case 9:
            dl_date_time=dl_date_time.addSecs(3600);
            break;
        case 10:
            dl_date_time=dl_date_time.addSecs(120);
            break;
        case 11:
            dl_date_time=dl_date_time.addSecs(300);
            break;
        case 12:
            dl_date_time=dl_date_time.addSecs(900);
            break;
        }
            QApplication::processEvents();
        }
    return true;
}

void WirelessModule2::get_data_clicked()
{
    pointeur_list_sensor_vidage=0;
    get_data();
}

void WirelessModule2::set_start_clicked()
{
    fillingSensorTable=true;
    QDateTime dl_start_t=start_date_sensor->dateTime();
    dl_start_t=dl_start_t.toLocalTime();
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x306,bcdFromDecimal(dl_start_t.date().year()%100),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x307,bcdFromDecimal(dl_start_t.date().month()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x308,bcdFromDecimal(dl_start_t.date().day()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x309,bcdFromDecimal(dl_start_t.time().hour()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30a,bcdFromDecimal(dl_start_t.time().minute()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30b,bcdFromDecimal(dl_start_t.time().second()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x156,2,3); //set start time
    waitFor(100);
    fillingSensorTable=false;
    refresh_values();
}

void WirelessModule2::dump_clicked()
{
    fillingSensorTable=true;
    QDateTime dl_start_t=date_vidage_sensor->dateTime();
    dl_start_t=dl_start_t.toLocalTime();
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000020,0,3); //clear pointer flash
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000021,0,3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000022,0,3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x306,bcdFromDecimal(dl_start_t.date().year()%100),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x307,bcdFromDecimal(dl_start_t.date().month()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x308,bcdFromDecimal(dl_start_t.date().day()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x309,bcdFromDecimal(dl_start_t.time().hour()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30a,bcdFromDecimal(dl_start_t.time().minute()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x30b,bcdFromDecimal(dl_start_t.time().second()),3);
    serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x156,8,3); //start dump
    fillingSensorTable=false;
}

bool WirelessModule2::get_data()
{
    uint32_t pointeur_flash;
    serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x01000020,1,3); //recupere pointeur flash
    pointeur_flash=serialCom->getReceivedData()[0]+(serialCom->getReceivedData()[1]<<8)+(serialCom->getReceivedData()[2]<<16);
    uint16_t pointeur_page=0;
    uint32_t current_index=0;
    uint32_t address=0;

    //Init the progress dialog
    QProgressDialog *downloadProgress = new QProgressDialog("Data transfer in progress...", "Cancel", 0, 100);
    downloadProgress->setWindowTitle("Downloading campaign...");
    downloadProgress->setWindowIcon(QIcon("icon/MVicon.png"));
    downloadProgress->setMinimum(0);    //%
    downloadProgress->setMaximum(100);
    downloadProgress->setWindowModality(Qt::ApplicationModal);
    downloadProgress->show();
    downloadProgress->setValue(0);

    do
    {
        address=current_index|0x03000000;
        //Reading values (64 uchar) :
        int n_try=3;
        bool ok=false;
        do
        {
            ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,address,8,5);
            if(serialCom->getReceivedDataLength()<64) ok=false;
            if (!ok){
                waitFor(100);
                //serialCom->deleteErrorCom();
            }
        }while(!ok&&n_try--);
        int response = QMessageBox::Retry;
        while(!ok && response==QMessageBox::Retry)
        {
            // In case of reading error, retry with a message window
            response=QMessageBox::warning(this,"Error during download","Reading address 0x"+QString::number(address,16)+", from Module SN "+QString::number(wireless_Module->serialNb,16).rightJustified(8,'0')+", failed.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
            if(response==QMessageBox::Retry)
            {
                //serialCom->deleteErrorCom();
                n_try=3;
                do
                {
                    ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,address,8,5);
                    if(serialCom->getReceivedDataLength()<64) ok=false;
                    if (!ok){
                        waitFor(100);
                        //serialCom->deleteErrorCom();
                    }
                }while(!ok&&n_try--);
            }
            else
            {
                downloadProgress->close();
                delete downloadProgress;
                return false;
            }
        }

        if(ok)
        {
            downloadProgress->activateWindow();
            //Write values in the table
            for(int tableIndex=0;tableIndex<64;tableIndex++)
            {
                if(pointeur_page<528) buffer_page[pointeur_page++]=serialCom->getReceivedData()[tableIndex];
                else
                {
                    updateFile();
                    pointeur_page=0;
                    buffer_page[pointeur_page++]=serialCom->getReceivedData()[tableIndex];
                }
            }


            // compute progress for QProgress display
            int percent = (100 * float(current_index))/pointeur_flash;
            if(percent>100) percent=100;
            downloadProgress->setValue(percent);
            if(downloadProgress->wasCanceled())
            {
                downloadProgress->close();
                delete downloadProgress;
                return false;
            }
            current_index+=64;
            QCoreApplication::processEvents();
        }// end of read data computing (return of readBytes)

    } while(current_index<pointeur_flash);
    downloadProgress->close();
    delete downloadProgress;

    for(int i=0;i<pointeur_list_sensor_vidage;i++)
    {
        tsvFile[i].close();
    }
    return true;
}

void WirelessModule2::backToManageModule()
{   //Button back clicked, stop refresh and return to module grid

    commonRefreshTimer->disconnectTimeout();
    emit backToMainWindow();    //Signal to restart the main window (refresh Values)
}
